# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os

from prompt_toolkit.completion import Completion

from awscli.autocomplete import completer, parser
from awscli.autocomplete.completer import CompletionResult
from awscli.autocomplete.local import basic
from awscli.testutils import mock, unittest
from tests.unit.autocomplete import InMemoryIndex


class TestAutoCompleter(unittest.TestCase):
    def setUp(self):
        self.parser = mock.Mock(spec=parser.CLIParser)
        self.parsed_result = parser.ParsedResult()
        self.parser.parse.return_value = self.parsed_result

    def test_delegates_to_autocompleters(self):
        mock_complete = mock.Mock(spec=completer.BaseCompleter)
        expected = [CompletionResult('ec2', -1), CompletionResult('ecs', -1)]
        mock_complete.complete.return_value = expected
        auto_complete = completer.AutoCompleter(
            self.parser, completers=[mock_complete]
        )

        results = auto_complete.autocomplete('aws e')
        self.assertEqual(results, expected)
        self.parser.parse.assert_called_with('aws e', None)
        mock_complete.complete.assert_called_with(self.parsed_result)

    def test_stops_processing_when_list_returned(self):
        first = mock.Mock(spec=completer.BaseCompleter)
        second = mock.Mock(spec=completer.BaseCompleter)

        first.complete.return_value = None
        expected = [CompletionResult('ec2', -1), CompletionResult('ecs', -1)]
        second.complete.return_value = expected

        auto_complete = completer.AutoCompleter(
            self.parser, completers=[first, second]
        )
        self.assertEqual(auto_complete.autocomplete('aws e'), expected)

        first.complete.assert_called_with(self.parsed_result)
        second.complete.assert_called_with(self.parsed_result)

    def test_returns_empty_list_if_no_completers_have_results(self):
        first = mock.Mock(spec=completer.BaseCompleter)
        second = mock.Mock(spec=completer.BaseCompleter)

        first.complete.return_value = None
        second.complete.return_value = None

        auto_complete = completer.AutoCompleter(
            self.parser, completers=[first, second]
        )
        self.assertEqual(auto_complete.autocomplete('aws e'), [])

        first.complete.assert_called_with(self.parsed_result)
        second.complete.assert_called_with(self.parsed_result)

    def test_first_result_wins(self):
        first = mock.Mock(spec=completer.BaseCompleter)
        second = mock.Mock(spec=completer.BaseCompleter)

        first.complete.return_value = [CompletionResult('ec2', -1)]
        second.complete.return_value = [CompletionResult('ecs', -1)]

        auto_complete = completer.AutoCompleter(
            self.parser, completers=[first, second]
        )
        self.assertEqual(
            auto_complete.autocomplete('aws e'), [CompletionResult('ec2', -1)]
        )

        first.complete.assert_called_with(self.parsed_result)
        self.assertFalse(second.complete.called)

    def test_strip_html_tags_and_newlines(self):
        STARTING_TOKEN_HELP = """
<p>A token to specify where to start paginating.  This is the
<code>NextToken</code> from a previously truncated response.</p>
"""
        help_text = basic.strip_html_tags_and_newlines(STARTING_TOKEN_HELP)
        self.assertEqual(
            help_text,
            'A token to specify where to start paginating.  This is the'
            'NextToken from a previously truncated response.',
        )


class TestModelIndexCompleter(unittest.TestCase):
    def setUp(self):
        self.index = InMemoryIndex(
            {
                'command_names': {
                    '': [('aws', None)],
                    'aws': [
                        ('ec2', 'Amazon Elastic Compute Cloud'),
                        ('ecs', 'Amazon EC2 Container Registry'),
                        ('s3', 'Amazon Simple Storage Service'),
                        ('s3api', 'Amazon Simple Storage Service'),
                    ],
                    'aws.ec2': [('describe-instances', None)],
                    'aws.s3api': [('get-object', None)],
                },
                'arg_names': {
                    '': {
                        'aws': ['region', 'endpoint-url'],
                    },
                    'aws.ec2': {
                        'describe-instances': [
                            'instance-ids',
                            'reserve',
                            'positional',
                        ],
                    },
                    'aws.s3api': {
                        'get-object': ['outfile', 'bucket', 'key'],
                    },
                },
                'arg_data': {
                    '': {
                        'aws': {
                            'endpoint-url': (
                                'endpoint-url',
                                'string',
                                'aws',
                                '',
                                None,
                                False,
                                False,
                            ),
                            'region': (
                                'region',
                                'string',
                                'aws',
                                '',
                                None,
                                False,
                                False,
                            ),
                        }
                    },
                    'aws.ec2': {
                        'describe-instances': {
                            'instance-ids': (
                                'instance-ids',
                                'string',
                                'describe-instances',
                                'aws.ec2.',
                                None,
                                False,
                                False,
                            ),
                            'reserve': (
                                'reserve',
                                'string',
                                'describe-instances',
                                'aws.ec2.',
                                None,
                                False,
                                False,
                            ),
                            'positional': (
                                'positional',
                                'string',
                                'describe-instances',
                                'aws.ec2.',
                                None,
                                True,
                                False,
                            ),
                        }
                    },
                    'aws.s3api': {
                        'get-object': {
                            'outfile': (
                                'outfile',
                                'string',
                                'get-object',
                                'aws.s3api.',
                                None,
                                False,
                                False,
                            ),
                            'bucket': (
                                'bucket',
                                'string',
                                'get-object',
                                'aws.s3api.',
                                None,
                                False,
                                False,
                            ),
                            'key': (
                                'key',
                                'string',
                                'get-object',
                                'aws.s3api.',
                                None,
                                False,
                                False,
                            ),
                        }
                    },
                },
            }
        )
        self.parser = parser.CLIParser(self.index)
        self.completer = basic.ModelIndexCompleter(self.index)

    def test_does_not_complete_if_unparsed_items(self):
        parsed = self.parser.parse('aws foo ')
        self.assertIsNone(self.completer.complete(parsed))

    def test_does_complete_if_current_fragment_is_none(self):
        parsed = self.parser.parse('aws')
        self.assertIsNone(self.completer.complete(parsed))

    def test_can_prefix_match_services(self):
        parsed = self.parser.parse('aws e')
        expected = [
            # The -1 is because we need to replace the string starting
            # 1 character back  (the last fragment is the string 'e').
            CompletionResult('ec2', starting_index=-1),
            CompletionResult('ecs', starting_index=-1),
        ]
        self.assertEqual(self.completer.complete(parsed), expected)

    def test_returns_all_results_when_current_fragment_empty(self):
        parsed = self.parser.parse('aws ')
        expected = [
            # The -1 is because we need to replace the string starting
            # 1 character back  (the last fragment is the string 'e').
            CompletionResult('ec2', starting_index=0),
            CompletionResult('ecs', starting_index=0),
            CompletionResult('s3', starting_index=0),
            CompletionResult('s3api', starting_index=0),
        ]
        self.assertEqual(self.completer.complete(parsed), expected)

    def test_can_autocomplete_global_param(self):
        parsed = self.parser.parse('aws --re')
        self.assertEqual(
            self.completer.complete(parsed), [CompletionResult('--region', -4)]
        )

    def test_can_autocomplete_dashes(self):
        parsed = self.parser.parse('aws --')
        self.assertTrue(len(self.completer.complete(parsed)) > 0)

    def test_keep_suggesting_if_there_are_longer_completions(self):
        parsed = self.parser.parse('aws s3')
        self.assertEqual(
            self.completer.complete(parsed),
            [
                CompletionResult(
                    's3',
                    -2,
                    False,
                    None,
                    'Amazon Simple Storage Service',
                ),
                CompletionResult(
                    's3api',
                    -2,
                    False,
                    None,
                    'Amazon Simple Storage Service',
                ),
            ],
        )

    def test_retain_option_order_on_dashes(self):
        parsed = self.parser.parse(
            'aws ec2 describe-instances --region ' 'us-west-2 --'
        )
        self.assertEqual(
            self.completer.complete(parsed),
            [
                CompletionResult(
                    '--instance-ids',
                    -2,
                    False,
                    'string',
                ),
                CompletionResult(
                    '--reserve',
                    -2,
                    False,
                    'string',
                ),
                CompletionResult(
                    '--endpoint-url',
                    -2,
                    False,
                    'string',
                ),
            ],
        )

    def test_can_combine_global_and_command_params(self):
        parsed = self.parser.parse('aws ec2 describe-instances --r')
        self.assertEqual(
            self.completer.complete(parsed),
            [
                CompletionResult(
                    '--reserve',
                    -3,
                    False,
                    'string',
                ),
                CompletionResult(
                    '--region',
                    -3,
                    False,
                    'string',
                ),
            ],
        )

    def test_not_suggest_entered_options(self):
        parsed = self.parser.parse(
            'aws ec2 describe-instances ' '--region us-west-2 --re'
        )
        self.assertEqual(
            self.completer.complete(parsed),
            [
                CompletionResult(
                    '--reserve',
                    -4,
                    False,
                    'string',
                )
            ],
        )

    def test_no_autocompletions_if_nothing_matches(self):
        parsed = self.parser.parse('aws --foo')
        self.assertEqual(self.completer.complete(parsed), [])

    def test_no_complete_positional_arguments(self):
        parsed = self.parser.parse('aws ec2 describe-instances --pos')
        self.assertEqual(self.completer.complete(parsed), [])

    def test_get_documentation_if_fetcher_provided(self):
        fake_cli_fetcher = mock.Mock()
        fake_cli_fetcher.get_argument_documentation.return_value = (
            '<p>Arg doc</p>'
        )
        fake_cli_fetcher.get_global_arg_documentation.return_value = (
            '<p>Global arg doc</p>'
        )
        completer = basic.ModelIndexCompleter(self.index, fake_cli_fetcher)
        parsed = self.parser.parse('aws ec2 describe-instances --r')
        self.assertEqual(
            completer.complete(parsed),
            [
                CompletionResult('--reserve', -3, False, 'string', 'Arg doc'),
                CompletionResult(
                    '--region', -3, False, 'string', 'Global arg doc'
                ),
            ],
        )

    def test_return_service_full_name(self):
        parsed = self.parser.parse('aws ec')
        self.assertEqual(
            self.completer.complete(parsed),
            [
                CompletionResult(
                    'ec2',
                    -2,
                    False,
                    None,
                    'Amazon Elastic Compute Cloud',
                    None,
                ),
                CompletionResult(
                    'ecs',
                    -2,
                    False,
                    None,
                    'Amazon EC2 Container Registry',
                    None,
                ),
            ],
        )

    def test_can_handle_outfile(self):
        parsed = self.parser.parse('aws s3api get-object filename ')
        self.assertEqual(
            self.completer.complete(parsed),
            [
                CompletionResult('--bucket', 0, False, 'string', None, None),
                CompletionResult('--key', 0, False, 'string', None, None),
                CompletionResult(
                    '--endpoint-url', 0, False, 'string', None, None
                ),
                CompletionResult('--region', 0, False, 'string', None, None),
            ],
        )

    def test_can_handle_outfile_with_options(self):
        parsed = self.parser.parse(
            'aws s3api get-object --bucket foo ' 'filename --region us-west-2 '
        )
        self.assertEqual(
            self.completer.complete(parsed),
            [
                CompletionResult('--key', 0, False, 'string', None, None),
                CompletionResult(
                    '--endpoint-url', 0, False, 'string', None, None
                ),
            ],
        )

    def test_can_suggest_outfile(self):
        parsed = self.parser.parse(
            'aws s3api get-object --bucket foo '
            '--key bar --region us-west-2 '
        )
        self.assertEqual(
            self.completer.complete(parsed),
            [
                CompletionResult(
                    'outfile', 0, False, 'string', None, 'outfile (required)'
                ),
                CompletionResult(
                    '--endpoint-url', 0, False, 'string', None, None
                ),
            ],
        )


class TestRegionCompleter(unittest.TestCase):
    def setUp(self):
        self.index = InMemoryIndex(
            {
                'command_names': {'': ['aws'], 'aws': []},
                'arg_names': {'': {'aws': ['region']}},
                'arg_data': {
                    '': {
                        'aws': {
                            'region': (
                                'region',
                                'string',
                                'aws',
                                '',
                                None,
                                False,
                                False,
                            ),
                        }
                    }
                },
            }
        )
        self.parser = parser.CLIParser(self.index)
        self.completer = basic.RegionCompleter()

    def test_return_none_if_it_is_not_region_option(self):
        parsed = self.parser.parse('aws --foo')
        self.assertIsNone(self.completer.complete(parsed))

    def test_no_autocompletions_if_nothing_matches(self):
        parsed = self.parser.parse('aws --region foo')
        self.assertEqual(list(self.completer.complete(parsed)), [])

    def test_autocompletes_if_matches(self):
        parsed = self.parser.parse('aws --region us-west-')
        self.assertEqual(
            list(self.completer.complete(parsed)),
            [
                CompletionResult('us-west-1', 0, False),
                CompletionResult('us-west-2', 0, False),
            ],
        )

    def test_not_autocomplete_wo_trailing_space(self):
        parsed = self.parser.parse('aws --region')
        self.assertIsNone(self.completer.complete(parsed))

    def test_autocomplete_with_trailing_space(self):
        parsed = self.parser.parse('aws --region ')
        self.assertIsNotNone(self.completer.complete(parsed))


class TestProfileCompleter(unittest.TestCase):
    def setUp(self):
        self.index = InMemoryIndex(
            {
                'command_names': {'': ['aws'], 'aws': []},
                'arg_names': {'': {'aws': ['profile']}},
                'arg_data': {
                    '': {
                        'aws': {
                            'profile': (
                                'profile',
                                'string',
                                'aws',
                                '',
                                None,
                                False,
                                False,
                            ),
                        }
                    }
                },
            }
        )
        fake_session = mock.Mock()
        fake_session.available_profiles = ['default', 'profile1', 'profile2']
        self.parser = parser.CLIParser(self.index)
        self.completer = basic.ProfileCompleter(session=fake_session)

    def test_return_none_if_it_is_not_profile_option(self):
        parsed = self.parser.parse('aws --foo')
        self.assertIsNone(self.completer.complete(parsed))

    def test_no_autocompletions_if_nothing_matches(self):
        parsed = self.parser.parse('aws --profile foo')
        self.assertEqual(list(self.completer.complete(parsed)), [])

    def test_not_autocomplete_wo_trailing_space(self):
        parsed = self.parser.parse('aws --profile')
        self.assertIsNone(self.completer.complete(parsed))

    def test_autocomplete_with_trailing_space(self):
        parsed = self.parser.parse('aws --profile ')
        self.assertIsNotNone(self.completer.complete(parsed))

    def test_autocompletes_if_matches(self):
        parsed = self.parser.parse('aws --profile prof')
        self.assertEqual(
            list(self.completer.complete(parsed)),
            [
                CompletionResult('profile1', 0, False),
                CompletionResult('profile2', 0, False),
            ],
        )


class TestFilePathCompleter(unittest.TestCase):
    def setUp(self):
        self.index = InMemoryIndex(
            {
                'command_names': {'': ['aws'], 'aws': []},
                'arg_names': {'': {'aws': ['profile']}},
                'arg_data': {
                    '': {
                        'aws': {
                            'profile': (
                                'profile',
                                'string',
                                'aws',
                                '',
                                None,
                                False,
                                False,
                            ),
                        }
                    }
                },
            }
        )
        fake_completer = mock.Mock()
        fake_completer.get_completions.return_value = [
            Completion(text='', display=[('', 'file')]),
            Completion(text='', display=[('', 'folder/')]),
        ]
        self.parser = parser.CLIParser(self.index)
        self.completer = basic.FilePathCompleter(path_completer=fake_completer)

    def test_return_none_if_it_is_not_file_prefix(self):
        parsed = self.parser.parse('aws --foo')
        self.assertIsNone(self.completer.complete(parsed))

    def test_complete_lonely_tilda_with_slash(self):
        parsed = self.parser.parse('aws --profile file://~')
        self.assertEqual(
            list(self.completer.complete(parsed)),
            [CompletionResult('file://~%s' % os.sep, 0, False)],
        )

    def test_complete_pass_correct_prefix(self):
        parsed = self.parser.parse('aws --profile fileb://')
        self.assertEqual(
            list(self.completer.complete(parsed)),
            [
                CompletionResult('fileb://file', 0, False, None, None, 'file'),
                CompletionResult(
                    'fileb://folder/', 0, False, None, None, 'folder/'
                ),
            ],
        )



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.autocomplete.local import model


class InMemoryIndex(model.ModelIndex):
    # An in-memory version of a model index.

    def __init__(self, index):
        """

        The index param you provide is a dictionary that has top level
        keys that correspond to the method names here:


        ::

            {'command_names': {'dot.lineage': [<values>]},
             'arg_names': {'dot.lineage': {'command_name': [<values>]}},
             'get_argument_data': {
                 'dot.lineage': {
                     'command-name': {
                         'arg-name': (argname, type, command, parent,
                                      nargs, positional_arg, required,
                                      help_text)
                     }
                 }
            }

        """
        self.index = index

    def command_names(self, lineage):
        return [row[0] for row in self.commands_with_full_name(lineage)]

    def commands_with_full_name(self, lineage):
        parent = '.'.join(lineage)
        return self.index['command_names'].get(parent, [])

    def arg_names(self, lineage, command_name, positional_arg=False):
        parent = '.'.join(lineage)
        arg_names = (
            self.index['arg_names'].get(parent, {}).get(command_name, [])
        )
        filtered_arg_names = []
        for arg_name in arg_names:
            arg_data = self.get_argument_data(lineage, command_name, arg_name)
            if arg_data.positional_arg == positional_arg:
                filtered_arg_names.append(arg_name)
        return filtered_arg_names

    def get_argument_data(self, lineage, command_name, arg_name):
        parent = '.'.join(lineage)
        arg_data = (
            self.index['arg_data']
            .get(parent, {})
            .get(command_name, {})
            .get(arg_name)
        )
        if arg_data is not None:
            return model.CLIArgument(*arg_data)

    def get_global_arg_data(self, lineage=[], command_name='aws'):
        parent = '.'.join(lineage)
        arg_data = self.index['arg_data'].get(parent, {}).get(command_name, {})
        results = []
        for _, value in arg_data.items():
            results.append(model.CLIArgument(*value))
        return results



# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import hashlib
import json
import os
import re

from awscli.testutils import mock
from tests.functional.sso import BaseSSOTest


class TestLoginCommand(BaseSSOTest):
    _TIMESTAMP_FORMAT_PATTERN = re.compile(
        r'\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\dZ'
    )

    def assert_cache_contains_registration(
        self, start_url, session_name, scopes, expected_client_id
    ):
        cached_files = os.listdir(self.token_cache_dir)

        cached_registration_filename = self._get_cached_registration_filename(
            start_url, session_name, scopes
        )

        self.assertIn(cached_registration_filename, cached_files)
        self.assertEqual(
            self._get_cached_response(cached_registration_filename)[
                'clientId'
            ],
            expected_client_id,
        )

    def assert_cache_contains_token(
        self,
        start_url,
        expected_token,
        session_name=None,
    ):
        cached_files = os.listdir(self.token_cache_dir)
        # The registration and cached access token
        self.assertEqual(len(cached_files), 2)
        cached_token_filename = self._get_cached_token_filename(
            start_url,
            session_name,
        )
        self.assertIn(cached_token_filename, cached_files)
        self.assertEqual(
            self._get_cached_response(cached_token_filename)['accessToken'],
            expected_token,
        )

    def _get_cached_registration_filename(
        self, start_url, session_name, scopes
    ):
        args = {
            'tool': 'botocore',
            'startUrl': start_url,
            'region': self.sso_region,
            'scopes': scopes,
            'session_name': session_name,
        }
        cache_args = json.dumps(args, sort_keys=True).encode('utf-8')
        return hashlib.sha1(cache_args).hexdigest() + '.json'

    def _get_cached_token_filename(self, start_url, session_name):
        to_hash = start_url
        if session_name:
            to_hash = session_name
        return hashlib.sha1(to_hash.encode('utf-8')).hexdigest() + '.json'

    def _get_cached_response(self, token_filename):
        token_path = os.path.join(self.token_cache_dir, token_filename)
        with open(token_path) as f:
            cached_response = json.loads(f.read())
            return cached_response

    def assert_cache_token_expiration_time_format_is_correct(self):
        token_filename = self._get_cached_token_filename(self.start_url, None)
        token_path = os.path.join(self.token_cache_dir, token_filename)
        with open(token_path) as f:
            cached_response = json.loads(f.read())
            self.assertIsNotNone(
                self._TIMESTAMP_FORMAT_PATTERN.match(
                    cached_response['expiresAt']
                )
            )

    def test_login_explicit_device(self):
        self.add_oidc_device_responses(self.access_token)
        self.run_cmd('sso login --use-device-code')
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_device_browser_handler_called_with(
            'foo',
            'https://sso.fake/device',
            'https://sso.verify',
        )
        self.assert_cache_contains_token(
            start_url=self.start_url, expected_token=self.access_token
        )

    def test_login_implicit_device(self):
        # This is a legacy profile via setUp, so we expect
        # it to fall back to device flow automatically
        self.add_oidc_device_responses(self.access_token)
        self.run_cmd('sso login')
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_device_browser_handler_called_with(
            'foo',
            'https://sso.fake/device',
            'https://sso.verify',
        )
        self.assert_cache_contains_token(
            start_url=self.start_url, expected_token=self.access_token
        )

    def test_login_device_no_browser(self):
        self.add_oidc_device_responses(self.access_token)
        stdout, _, _ = self.run_cmd('sso login --use-device-code --no-browser')
        self.assertIn('Browser will not be automatically opened.', stdout)
        self.open_browser_mock.assert_not_called()
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_cache_contains_token(
            start_url=self.start_url, expected_token=self.access_token
        )

    def test_login_auth_no_browser(self):
        content = self.get_sso_session_config('test-session')
        self.set_config_file_content(content=content)
        self.add_oidc_auth_code_responses(self.access_token)
        stdout, _, _ = self.run_cmd('sso login --no-browser')
        self.assertIn('Browser will not be automatically opened.', stdout)
        self.open_browser_mock.assert_not_called()
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_cache_contains_registration(
            start_url=self.start_url,
            session_name='test-session',
            scopes=self.registration_scopes,
            expected_client_id='auth-client-id',
        )
        self.assert_cache_contains_token(
            start_url=self.start_url,
            expected_token=self.access_token,
            session_name='test-session',
        )

    def test_login_device_forces_refresh(self):
        self.add_oidc_device_responses(self.access_token)
        self.run_cmd('sso login --use-device-code')
        # The register response from the first login should have been
        # cached.
        self.add_oidc_device_responses(
            'new.token', include_register_response=False
        )
        self.run_cmd('sso login --use-device-code')
        self.assert_cache_contains_token(
            start_url=self.start_url,
            expected_token='new.token',
        )

    def test_login_auth_forces_refresh(self):
        content = self.get_sso_session_config('test-session')
        self.set_config_file_content(content=content)
        self.add_oidc_auth_code_responses(self.access_token)
        self.run_cmd('sso login')
        # The register response from the first login should have been
        # cached.
        self.add_oidc_auth_code_responses(
            'new.token', include_register_response=False
        )
        self.run_cmd('sso login')
        self.assert_cache_contains_token(
            start_url=self.start_url,
            expected_token='new.token',
            session_name='test-session',
        )

    def test_login_auth_after_device_forces_refresh(self):
        self.set_config_file_content(
            content=self.get_sso_session_config('test-session')
        )
        self.add_oidc_device_responses(self.access_token)
        self.run_cmd('sso login --use-device-code')
        # The register response from the first login should have been
        # cached.
        self.add_oidc_auth_code_responses('new.token')
        self.run_cmd('sso login')
        self.assert_cache_contains_registration(
            start_url=self.start_url,
            session_name='test-session',
            scopes=self.registration_scopes,
            expected_client_id='auth-client-id',
        )
        self.assert_cache_contains_token(
            start_url=self.start_url,
            expected_token='new.token',
            session_name='test-session',
        )

    def test_login_device_no_sso_configuration(self):
        self.set_config_file_content(content='')
        _, stderr, _ = self.run_cmd(
            'sso login --use-device-code', expected_rc=253
        )
        self.assertIn(
            'Missing the following required SSO configuration', stderr
        )

    def test_login_auth_no_sso_configuration(self):
        self.set_config_file_content(content='')
        _, stderr, _ = self.run_cmd('sso login', expected_rc=253)
        self.assertIn(
            'Missing the following required SSO configuration', stderr
        )

    def test_login_device_minimal_sso_configuration(self):
        content = (
            '[default]\n'
            f'sso_start_url={self.start_url}\n'
            f'sso_region={self.sso_region}\n'
        )
        self.set_config_file_content(content=content)
        self.add_oidc_device_responses(self.access_token)
        self.run_cmd('sso login')
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_device_browser_handler_called_with(
            'foo',
            'https://sso.fake/device',
            'https://sso.verify',
        )
        self.assert_cache_contains_token(
            start_url=self.start_url, expected_token=self.access_token
        )

    def test_login_device_partially_missing_sso_configuration(self):
        content = '[default]\n' 'sso_start_url=%s\n' % self.start_url
        self.set_config_file_content(content=content)
        _, stderr, _ = self.run_cmd(
            'sso login --use-device-code', expected_rc=253
        )
        self.assertIn(
            'Missing the following required SSO configuration', stderr
        )
        self.assertIn('sso_region', stderr)
        self.assertNotIn('sso_start_url', stderr)
        self.assertNotIn('sso_account_id', stderr)
        self.assertNotIn('sso_role_name', stderr)

    def test_token_cache_datetime_format(self):
        self.add_oidc_device_responses(self.access_token)
        self.run_cmd('sso login --use-device-code')
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_cache_contains_token(
            start_url=self.start_url, expected_token=self.access_token
        )
        self.assert_cache_token_expiration_time_format_is_correct()

    def test_login_device_sso_session(self):
        content = self.get_sso_session_config('test-session')
        self.set_config_file_content(content=content)
        self.add_oidc_device_responses(self.access_token)
        self.run_cmd('sso login --use-device-code')
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_device_browser_handler_called_with(
            'foo',
            'https://sso.fake/device',
            'https://sso.verify',
        )
        self.assert_cache_contains_registration(
            start_url=self.start_url,
            session_name='test-session',
            scopes=self.registration_scopes,
            expected_client_id='device-client-id',
        )
        self.assert_cache_contains_token(
            start_url=self.start_url,
            session_name='test-session',
            expected_token=self.access_token,
        )

    def test_login_auth_sso_session(self):
        content = self.get_sso_session_config('test-session')
        self.set_config_file_content(content=content)
        self.add_oidc_auth_code_responses(self.access_token)
        self.run_cmd('sso login')
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_auth_browser_handler_called_with('sso%3Aaccount%3Aaccess')
        self.assert_cache_contains_registration(
            start_url=self.start_url,
            session_name='test-session',
            scopes=self.registration_scopes,
            expected_client_id='auth-client-id',
        )
        self.assert_cache_contains_token(
            start_url=self.start_url,
            session_name='test-session',
            expected_token=self.access_token,
        )

    def test_login_device_sso_with_explicit_sso_session_arg(self):
        content = self.get_sso_session_config(
            'test-session', include_profile=False
        )
        self.set_config_file_content(content=content)
        self.add_oidc_device_responses(self.access_token)
        self.run_cmd('sso login --sso-session test-session --use-device-code')
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_device_browser_handler_called_with(
            'foo',
            'https://sso.fake/device',
            'https://sso.verify',
        )
        self.assert_cache_contains_registration(
            start_url=self.start_url,
            session_name='test-session',
            scopes=self.registration_scopes,
            expected_client_id='device-client-id',
        )
        self.assert_cache_contains_token(
            start_url=self.start_url,
            session_name='test-session',
            expected_token=self.access_token,
        )

    def test_login_auth_sso_with_explicit_sso_session_arg(self):
        content = self.get_sso_session_config(
            'test-session', include_profile=False
        )
        self.set_config_file_content(content=content)
        self.add_oidc_auth_code_responses(self.access_token)
        self.run_cmd('sso login --sso-session test-session')
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_auth_browser_handler_called_with('sso%3Aaccount%3Aaccess')
        self.assert_cache_contains_registration(
            start_url=self.start_url,
            session_name='test-session',
            scopes=self.registration_scopes,
            expected_client_id='auth-client-id',
        )
        self.assert_cache_contains_token(
            start_url=self.start_url,
            session_name='test-session',
            expected_token=self.access_token,
        )

    def test_login_device_sso_session_with_scopes(self):
        self.registration_scopes = ['sso:foo', 'sso:bar']
        content = self.get_sso_session_config('test-session')
        self.set_config_file_content(content=content)
        self.add_oidc_device_responses(self.access_token)
        self.run_cmd('sso login --use-device-code')
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_device_browser_handler_called_with(
            'foo',
            'https://sso.fake/device',
            'https://sso.verify',
        )
        self.assert_cache_contains_registration(
            start_url=self.start_url,
            session_name='test-session',
            scopes=self.registration_scopes,
            expected_client_id='device-client-id',
        )
        self.assert_cache_contains_token(
            start_url=self.start_url,
            session_name='test-session',
            expected_token=self.access_token,
        )
        operation, params = self.operations_called[0]
        self.assertEqual(operation.name, 'RegisterClient')
        self.assertEqual(params.get('scopes'), self.registration_scopes)

    def test_login_auth_sso_session_with_scopes(self):
        self.registration_scopes = ['sso:foo', 'sso:bar']
        content = self.get_sso_session_config('test-session')
        self.set_config_file_content(content=content)
        self.add_oidc_auth_code_responses(self.access_token)
        self.run_cmd('sso login')
        self.assert_used_expected_sso_region(expected_region=self.sso_region)
        self.assert_auth_browser_handler_called_with('sso%3Afoo%20sso%3Abar')
        self.assert_cache_contains_registration(
            start_url=self.start_url,
            session_name='test-session',
            scopes=self.registration_scopes,
            expected_client_id='auth-client-id',
        )
        self.assert_cache_contains_token(
            start_url=self.start_url,
            session_name='test-session',
            expected_token=self.access_token,
        )
        operation, params = self.operations_called[0]
        self.assertEqual(operation.name, 'RegisterClient')
        self.assertEqual(params.get('scopes'), self.registration_scopes)

    def test_login_sso_session_missing_config(self):
        content = '[default]\n' 'sso_session=test\n' '[sso-session test]\n'
        self.set_config_file_content(content=content)
        _, stderr, _ = self.run_cmd('sso login', expected_rc=253)
        self.assertIn(
            'SSO configuration values: sso_start_url, sso_region', stderr
        )

    def test_login_sso_session_missing(self):
        content = '[default]\n' 'sso_session=test\n'
        self.set_config_file_content(content=content)
        _, stderr, _ = self.run_cmd('sso login', expected_rc=253)
        self.assertIn('sso-session does not exist: "test"', stderr)

    def test_login_auth_sso_no_authorization_code_throws_error(self):
        self.fetcher_mock.return_value.get_auth_code_and_state.return_value = (
            None,
            None,
        )
        content = self.get_sso_session_config('test-session')
        self.set_config_file_content(content=content)
        self.add_oidc_auth_code_responses(self.access_token)

        _, stderr, _ = self.run_cmd('sso login', expected_rc=255)
        self.assertIn('Failed to retrieve an authorization code.', stderr)

    def test_login_auth_sso_state_mismatch_throws_error(self):
        self.fetcher_mock.return_value.get_auth_code_and_state.return_value = (
            "abc",
            '00000000-0000-0000-0000-000000000001',
        )
        content = self.get_sso_session_config('test-session')
        self.set_config_file_content(content=content)
        self.add_oidc_auth_code_responses(self.access_token)

        _, stderr, _ = self.run_cmd('sso login', expected_rc=255)
        self.assertIn('State parameter does not match expected value.', stderr)

    def test_login_device_no_extra_user_agent(self):
        self.add_oidc_device_responses(self.access_token)
        self.run_cmd('sso login --use-device-code')
        self.assertNotIn(
            'md/sso#auth', self.last_request_dict['headers']['User-Agent']
        )

    def test_login_auth_includes_extra_user_agent(self):
        content = self.get_sso_session_config('test-session')
        self.set_config_file_content(content=content)
        self.add_oidc_auth_code_responses(self.access_token)
        self.run_cmd('sso login')
        self.assertIn(
            'md/sso#auth', self.last_request_dict['headers']['User-Agent']
        )

    @mock.patch('botocore.session.Session.create_client')
    def test_login_custom_ca(self, create_client_patch):
        # Profile and mock responses aren't wired up so this fails with 255,
        # but it does get far enough to verify that the internal SSO-OIDC
        # client is created with the custom CA bundle
        self.run_cmd(
            'sso login --ca-bundle /path/to/ca/bundle.pem', expected_rc=255
        )
        create_client_patch.assert_called_once_with(
            'sso-oidc', config=mock.ANY, verify='/path/to/ca/bundle.pem'
        )

    @mock.patch('botocore.session.Session.create_client')
    def test_login_no_verify_ssl(self, create_client_patch):
        # Profile and mock responses aren't wired up so this fails with 255,
        # but it does get far enough to verify that the internal SSO-OIDC
        # client is created with verify=False
        self.run_cmd('sso login --no-verify-ssl', expected_rc=255)
        create_client_patch.assert_called_once_with(
            'sso-oidc',
            config=mock.ANY,
            verify=False,
        )



# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from botocore import hooks
from botocore.exceptions import (
    ParamValidationError,
    StubResponseError,
    UnStubbedResponseError,
)
from botocore.model import ServiceModel
from botocore.stub import Stubber

from tests import mock, unittest


class TestStubber(unittest.TestCase):
    def setUp(self):
        self.event_emitter = hooks.HierarchicalEmitter()
        self.client = mock.Mock()
        self.client.meta.events = self.event_emitter
        self.client.meta.method_to_api_mapping.get.return_value = 'foo'
        self.stubber = Stubber(self.client)
        self.validate_parameters_mock = mock.Mock()
        self.validate_parameters_patch = mock.patch(
            'botocore.stub.validate_parameters', self.validate_parameters_mock
        )
        self.validate_parameters_patch.start()

    def tearDown(self):
        self.validate_parameters_patch.stop()

    def emit_get_response_event(
        self, model=None, request_dict=None, signer=None, context=None
    ):
        if model is None:
            model = mock.Mock()
            model.name = 'foo'

        handler, response = self.event_emitter.emit_until_response(
            event_name='before-call.myservice.foo',
            model=model,
            params=request_dict,
            request_signer=signer,
            context=context,
        )

        return response

    def test_stubber_registers_events(self):
        self.event_emitter = mock.Mock()
        self.client.meta.events = self.event_emitter
        self.stubber.activate()
        # This just ensures that we register at the correct event
        # and nothing more
        self.event_emitter.register_first.assert_called_with(
            'before-parameter-build.*.*', mock.ANY, unique_id=mock.ANY
        )
        self.event_emitter.register.assert_called_with(
            'before-call.*.*', mock.ANY, unique_id=mock.ANY
        )

    def test_stubber_unregisters_events(self):
        self.event_emitter = mock.Mock()
        self.client.meta.events = self.event_emitter
        self.stubber.activate()
        self.stubber.deactivate()
        self.event_emitter.unregister.assert_any_call(
            'before-parameter-build.*.*', mock.ANY, unique_id=mock.ANY
        )
        self.event_emitter.unregister.assert_any_call(
            'before-call.*.*', mock.ANY, unique_id=mock.ANY
        )

    def test_context_manager(self):
        self.event_emitter = mock.Mock()
        self.client.meta.events = self.event_emitter

        with self.stubber:
            # Ensure events are registered in context
            self.event_emitter.register_first.assert_called_with(
                'before-parameter-build.*.*', mock.ANY, unique_id=mock.ANY
            )
            self.event_emitter.register.assert_called_with(
                'before-call.*.*', mock.ANY, unique_id=mock.ANY
            )

        # Ensure events are no longer registered once we leave the context
        self.event_emitter.unregister.assert_any_call(
            'before-parameter-build.*.*', mock.ANY, unique_id=mock.ANY
        )
        self.event_emitter.unregister.assert_any_call(
            'before-call.*.*', mock.ANY, unique_id=mock.ANY
        )

    def test_add_response(self):
        response = {'foo': 'bar'}
        self.stubber.add_response('foo', response)

        with self.assertRaises(AssertionError):
            self.stubber.assert_no_pending_responses()

    def test_add_response_fails_when_missing_client_method(self):
        del self.client.foo
        with self.assertRaises(ValueError):
            self.stubber.add_response('foo', {})

    def test_validates_service_response(self):
        self.stubber.add_response('foo', {})
        self.assertTrue(self.validate_parameters_mock.called)

    def test_validate_ignores_response_metadata(self):
        service_response = {'ResponseMetadata': {'foo': 'bar'}}
        service_model = ServiceModel(
            {
                'documentation': '',
                'operations': {
                    'foo': {
                        'name': 'foo',
                        'input': {'shape': 'StringShape'},
                        'output': {'shape': 'StringShape'},
                    }
                },
                'shapes': {'StringShape': {'type': 'string'}},
            }
        )
        op_name = service_model.operation_names[0]
        output_shape = service_model.operation_model(op_name).output_shape

        self.client.meta.service_model = service_model
        self.stubber.add_response('TestOperation', service_response)
        self.validate_parameters_mock.assert_called_with({}, output_shape)

        # Make sure service response hasn't been mutated
        self.assertEqual(
            service_response, {'ResponseMetadata': {'foo': 'bar'}}
        )

    def test_validates_on_empty_output_shape(self):
        service_model = ServiceModel(
            {'documentation': '', 'operations': {'foo': {'name': 'foo'}}}
        )
        self.client.meta.service_model = service_model

        with self.assertRaises(ParamValidationError):
            self.stubber.add_response('TestOperation', {'foo': 'bar'})

    def test_get_response(self):
        service_response = {'bar': 'baz'}
        self.stubber.add_response('foo', service_response)
        self.stubber.activate()
        response = self.emit_get_response_event()
        self.assertEqual(response[1], service_response)
        self.assertEqual(response[0].status_code, 200)

    def test_get_client_error_response(self):
        error_code = "foo"
        service_message = "bar"
        self.stubber.add_client_error('foo', error_code, service_message)
        self.stubber.activate()
        response = self.emit_get_response_event()
        self.assertEqual(response[1]['Error']['Message'], service_message)
        self.assertEqual(response[1]['Error']['Code'], error_code)

    def test_get_client_error_with_extra_error_meta(self):
        error_code = "foo"
        error_message = "bar"
        error_meta = {
            "Endpoint": "https://foo.bar.baz",
        }
        self.stubber.add_client_error(
            'foo',
            error_code,
            error_message,
            http_status_code=301,
            service_error_meta=error_meta,
        )
        with self.stubber:
            response = self.emit_get_response_event()
        error = response[1]['Error']
        self.assertIn('Endpoint', error)
        self.assertEqual(error['Endpoint'], "https://foo.bar.baz")

    def test_get_client_error_with_extra_response_meta(self):
        error_code = "foo"
        error_message = "bar"
        stub_response_meta = {
            "RequestId": "79104EXAMPLEB723",
        }
        self.stubber.add_client_error(
            'foo',
            error_code,
            error_message,
            http_status_code=301,
            response_meta=stub_response_meta,
        )
        with self.stubber:
            response = self.emit_get_response_event()
        actual_response_meta = response[1]['ResponseMetadata']
        self.assertIn('RequestId', actual_response_meta)
        self.assertEqual(actual_response_meta['RequestId'], "79104EXAMPLEB723")

    def test_get_response_errors_with_no_stubs(self):
        self.stubber.activate()
        with self.assertRaises(UnStubbedResponseError):
            self.emit_get_response_event()

    def test_assert_no_responses_remaining(self):
        self.stubber.add_response('foo', {})
        with self.assertRaises(AssertionError):
            self.stubber.assert_no_pending_responses()



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os

from awscli.customizations.arguments import (
    NestedBlobArgumentHoister,
    OverrideRequiredArgsArgument,
    QueryOutFileArgument,
    StatefulArgument,
    is_parsed_result_successful,
    resolve_given_outfile_path,
)
from awscli.customizations.exceptions import ParamValidationError
from awscli.testutils import FileCreator, mock, skip_if_windows, unittest


class TestOverrideRequiredArgsArgument(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock()
        self.argument = OverrideRequiredArgsArgument(self.session)

        # Set up a sample argument_table
        self.argument_table = {}
        self.mock_arg = mock.Mock()
        self.mock_arg.required = True
        self.argument_table['mock-arg'] = self.mock_arg

    def test_register_argument_action(self):
        register_args = self.session.register.call_args
        self.assertEqual(
            register_args[0][0], 'before-building-argument-table-parser'
        )
        self.assertEqual(
            register_args[0][1], self.argument.override_required_args
        )

    def test_override_required_args_if_in_cmdline(self):
        args = ['--no-required-args']
        self.argument.override_required_args(self.argument_table, args)
        self.assertFalse(self.mock_arg.required)

    def test_no_override_required_args_if_not_in_cmdline(self):
        args = []
        self.argument.override_required_args(self.argument_table, args)
        self.assertTrue(self.mock_arg.required)


class TestStatefulArgument(unittest.TestCase):
    def test_persists_value_when_added_to_params(self):
        self.session = mock.Mock()
        arg = StatefulArgument('test')
        arg.add_to_params({}, 'foo')
        self.assertEqual('foo', arg.value)


class TestArgumentHelpers(unittest.TestCase):
    def setUp(self):
        self.files = FileCreator()

    def tearDown(self):
        self.files.remove_all()

    def test_only_validates_filename_when_set(self):
        resolve_given_outfile_path(None)

    def test_works_with_valid_filename(self):
        filename = self.files.create_file('valid', '')
        self.assertEqual(filename, resolve_given_outfile_path(filename))

    def test_works_with_relative_filename(self):
        filename = '../valid'
        self.assertEqual(filename, resolve_given_outfile_path(filename))

    def test_raises_when_cannot_write_to_file(self):
        filename = os.sep.join(['_path', 'not', '_exist_', 'file.xyz'])
        with self.assertRaises(ParamValidationError):
            resolve_given_outfile_path(filename)

    def test_checks_if_valid_result(self):
        result = {'ResponseMetadata': {'HTTPStatusCode': 200}}
        self.assertTrue(is_parsed_result_successful(result))

    def test_checks_if_invalid_result(self):
        result = {'ResponseMetadata': {'HTTPStatusCode': 300}}
        self.assertFalse(is_parsed_result_successful(result))


class TestQueryFileArgument(unittest.TestCase):
    def setUp(self):
        self.files = FileCreator()

    def tearDown(self):
        self.files.remove_all()

    def test_proxies_to_super_ctor(self):
        session = mock.Mock()
        arg = QueryOutFileArgument(session, 'foo', 'bar.baz', 'event', 0o600)
        self.assertEqual('foo', arg.name)
        self.assertEqual('bar.baz', arg.query)

    def test_adds_default_help_text(self):
        session = mock.Mock()
        arg = QueryOutFileArgument(session, 'foo', 'bar.baz', 'event', 0o600)
        self.assertEqual(
            (
                'Saves the command output contents of bar.baz '
                'to the given filename'
            ),
            arg.documentation,
        )

    def test_does_not_add_help_text_if_set(self):
        session = mock.Mock()
        arg = QueryOutFileArgument(
            session, 'foo', 'bar.baz', 'event', 0o600, help_text='abc'
        )
        self.assertEqual('abc', arg.documentation)

    def test_saves_query_to_file(self):
        outfile = self.files.create_file('not-empty-test', '')
        session = mock.Mock()
        arg = QueryOutFileArgument(session, 'foo', 'baz', 'event', 0o600)
        arg.add_to_params({}, outfile)
        arg.save_query(
            {'ResponseMetadata': {'HTTPStatusCode': 200}, 'baz': 'abc123'}
        )
        with open(outfile) as fp:
            self.assertEqual('abc123', fp.read())
        self.assertEqual(1, session.register.call_count)
        session.register.assert_called_with('event', arg.save_query)

    def test_does_not_save_when_not_set(self):
        session = mock.Mock()
        QueryOutFileArgument(session, 'foo', 'baz', 'event', 0o600)
        self.assertEqual(0, session.register.call_count)

    def test_saves_query_to_file_as_empty_string_when_none_result(self):
        outfile = self.files.create_file('none-test', '')
        session = mock.Mock()
        arg = QueryOutFileArgument(session, 'foo', 'baz', 'event', 0o600)
        arg.add_to_params({}, outfile)
        arg.save_query({'ResponseMetadata': {'HTTPStatusCode': 200}})
        with open(outfile) as fp:
            self.assertEqual('', fp.read())

    @skip_if_windows("Test not valid on windows.")
    def test_permissions_on_created_file(self):
        outfile = self.files.create_file('not-empty-test', '')
        session = mock.Mock()
        arg = QueryOutFileArgument(session, 'foo', 'baz', 'event', 0o600)
        arg.add_to_params({}, outfile)
        arg.save_query(
            {'ResponseMetadata': {'HTTPStatusCode': 200}, 'baz': 'abc123'}
        )
        with open(outfile) as fp:
            fp.read()
        self.assertEqual(os.stat(outfile).st_mode & 0xFFF, 0o600)


class TestNestedBlobArgumentHoister(unittest.TestCase):
    def setUp(self):
        self.blob_hoister = NestedBlobArgumentHoister(
            'complexArgX', 'valueY', 'argY', 'argYDoc', '.argYDocAddendum'
        )

        self.arg_table = {
            'complexArgX': mock.Mock(
                required=True,
                documentation='complexArgXDoc',
                argument_model=mock.Mock(
                    members={
                        'valueY': mock.Mock(
                            type_name='blob',
                        )
                    }
                ),
            )
        }

    def test_apply_to_arg_table(self):
        self.blob_hoister(None, self.arg_table)

        self.assertFalse(self.arg_table['complexArgX'].required)
        self.assertEqual(
            self.arg_table['complexArgX'].documentation,
            'complexArgXDoc.argYDocAddendum',
        )

        argY = self.arg_table['argY']
        self.assertFalse(argY.required)
        self.assertEqual(argY.documentation, 'argYDoc')
        self.assertEqual(argY.argument_model.type_name, 'blob')

    def test_populates_underlying_complex_arg(self):
        self.blob_hoister(None, self.arg_table)
        argY = self.arg_table['argY']

        # parameters bag doesn't
        # already contain 'ComplexArgX'
        parameters = {
            'any': 'other',
        }
        argY.add_to_params(parameters, 'test-value')
        self.assertEqual('other', parameters['any'])
        self.assertEqual('test-value', parameters['ComplexArgX']['valueY'])

    def test_preserves_member_values_in_underlying_complex_arg(self):
        self.blob_hoister(None, self.arg_table)
        argY = self.arg_table['argY']

        # parameters bag already contains 'ComplexArgX'
        # but that does not contain 'valueY'
        parameters = {
            'any': 'other',
            'ComplexArgX': {
                'another': 'one',
            },
        }
        argY.add_to_params(parameters, 'test-value')
        self.assertEqual('other', parameters['any'])
        self.assertEqual('one', parameters['ComplexArgX']['another'])
        self.assertEqual('test-value', parameters['ComplexArgX']['valueY'])

    def test_overrides_target_member_in_underlying_complex_arg(self):
        self.blob_hoister(None, self.arg_table)
        argY = self.arg_table['argY']

        # parameters bag already contains 'ComplexArgX'
        # and that already contains 'valueY'
        parameters = {
            'any': 'other',
            'ComplexArgX': {
                'another': 'one',
                'valueY': 'doomed',
            },
        }
        argY.add_to_params(parameters, 'test-value')
        self.assertEqual('other', parameters['any'])
        self.assertEqual('one', parameters['ComplexArgX']['another'])
        self.assertEqual('test-value', parameters['ComplexArgX']['valueY'])

    def test_not_apply_to_mismatch_arg_type(self):
        nonmatching_arg_table = {
            'complexArgX': mock.Mock(
                required=True,
                documentation='complexArgXDoc',
                argument_model=mock.Mock(
                    members={
                        'valueY': mock.Mock(
                            type_name='string',
                        )
                    }
                ),
            )
        }

        self.blob_hoister(None, nonmatching_arg_table)

        self.assertTrue(nonmatching_arg_table['complexArgX'].required)
        self.assertEqual(
            nonmatching_arg_table['complexArgX'].documentation,
            'complexArgXDoc',
        )
        self.assertFalse('argY' in self.arg_table)



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import logging
import os

from awscli.clidriver import create_clidriver
from awscli.testutils import BaseAWSCommandParamsTest, FileCreator, mock

logger = logging.getLogger(__name__)


class BaseTestCLIFollowParamFile(BaseAWSCommandParamsTest):
    def setUp(self):
        super(BaseTestCLIFollowParamFile, self).setUp()
        self.files = FileCreator()
        self.prefix = 'lambda get-function --function-name'

    def tearDown(self):
        super(BaseTestCLIFollowParamFile, self).tearDown()
        self.files.remove_all()

    def assert_param_expansion_is_correct(
        self, provided_param, expected_param
    ):
        cmd = '%s %s' % (self.prefix, provided_param)
        # We do not care about the return code here. All that is of interest
        # is what happened to the arguments before they were passed to botocore
        # which we get from the params={} key. For binary types we will fail in
        # python 3 with an rc of 255 and get an rc of 0 in python 2 where it
        # can't tell the difference, so we pass ANY here to ignore the rc.
        self.assert_params_for_cmd(
            cmd, params={'FunctionName': expected_param}, expected_rc=mock.ANY
        )


class TestCLIFollowParamFileDefault(BaseTestCLIFollowParamFile):
    def test_does_not_prefixes_when_none_in_param(self):
        self.assert_param_expansion_is_correct(
            provided_param='foobar', expected_param='foobar'
        )

    def test_does_use_file_prefix(self):
        path = self.files.create_file('foobar.txt', 'file content')
        param = 'file://%s' % path
        self.assert_param_expansion_is_correct(
            provided_param=param, expected_param='file content'
        )

    def test_does_use_fileb_prefix(self):
        # The command will fail with error code 255 since bytes type does
        # not work for this parameter, however we still record the raw
        # parameter that we passed, which is all this test is concerend about.
        path = self.files.create_file('foobar.txt', b'file content', mode='wb')
        param = 'fileb://%s' % path
        self.assert_param_expansion_is_correct(
            provided_param=param, expected_param=b'file content'
        )


class TestCLIUseEncodingFromEnv(BaseTestCLIFollowParamFile):
    def setUp(self):
        super(TestCLIUseEncodingFromEnv, self).setUp()
        self.path = self.files.create_file(
            'foobar.txt', '經理', encoding='utf-8'
        )

    def test_does_use_encoding_utf8(self):
        self.environ['AWS_CLI_FILE_ENCODING'] = 'utf-8'
        param = 'file://%s' % self.path
        self.assert_param_expansion_is_correct(
            provided_param=param, expected_param='經理'
        )

    def test_does_use_encoding_cp1251(self):
        self.environ['AWS_CLI_FILE_ENCODING'] = 'cp1251'
        param = 'file://%s' % self.path
        self.assert_param_expansion_is_correct(
            provided_param=param, expected_param='з¶“зђ†'
        )



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import argparse
import os
import shutil

from awscli_venv import AwsCliVenv
from constants import (
    BUILD_DIR,
    INSTALL_DIRNAME,
    ArtifactType,
)
from install import (
    Installer,
    Uninstaller,
)
from validate_env import validate_env

from exe import ExeBuilder


def create_exe(aws_venv, build_dir):
    exe_workspace = os.path.join(build_dir, "exe")
    if os.path.exists(exe_workspace):
        shutil.rmtree(exe_workspace)
    builder = ExeBuilder(exe_workspace, aws_venv)
    builder.build()


def build(parsed_args):
    aws_venv = _bootstap_venv(
        parsed_args.build_dir,
        parsed_args.artifact,
        parsed_args.download_deps,
    )
    if parsed_args.artifact == ArtifactType.PORTABLE_EXE.value:
        create_exe(aws_venv, parsed_args.build_dir)


def validate(parsed_args):
    validate_env(parsed_args.artifact)


def install(parsed_args):
    build_dir = parsed_args.build_dir
    install_dir = os.path.join(parsed_args.lib_dir, INSTALL_DIRNAME)
    bin_dir = parsed_args.bin_dir
    installer = Installer(build_dir)
    installer.install(install_dir, bin_dir)


def uninstall(parsed_args):
    install_dir = os.path.join(parsed_args.lib_dir, INSTALL_DIRNAME)
    bin_dir = parsed_args.bin_dir
    uninstaller = Uninstaller()
    uninstaller.uninstall(install_dir, bin_dir)


def _bootstap_venv(build_dir: str, artifact_type: str, download_deps: bool):
    venv_path = os.path.join(build_dir, "venv")
    if os.path.exists(venv_path):
        shutil.rmtree(venv_path)
    os.makedirs(venv_path)
    aws_venv = AwsCliVenv(venv_path)
    aws_venv.create()
    aws_venv.bootstrap(artifact_type, download_deps)
    return aws_venv


def main():
    parser = argparse.ArgumentParser()

    subparser = parser.add_subparsers()

    validate_env_parser = subparser.add_parser("validate-env")
    validate_env_parser.add_argument(
        "--artifact", choices=[e.value for e in ArtifactType], required=True
    )
    validate_env_parser.set_defaults(func=validate)

    build_parser = subparser.add_parser("build")
    build_parser.add_argument(
        "--artifact", choices=[e.value for e in ArtifactType], required=True
    )
    build_parser.add_argument(
        "--build-dir", default=BUILD_DIR, type=os.path.abspath
    )
    build_parser.add_argument("--download-deps", action="store_true")
    build_parser.set_defaults(func=build)

    install_parser = subparser.add_parser("install")
    install_parser.add_argument(
        "--build-dir", default=BUILD_DIR, type=os.path.abspath
    )
    install_parser.add_argument(
        "--lib-dir", required=True, type=os.path.abspath
    )
    install_parser.add_argument(
        "--bin-dir", required=True, type=os.path.abspath
    )
    install_parser.set_defaults(func=install)

    uninstall_parser = subparser.add_parser("uninstall")
    uninstall_parser.add_argument(
        "--lib-dir", required=True, type=os.path.abspath
    )
    uninstall_parser.add_argument(
        "--bin-dir", required=True, type=os.path.abspath
    )
    uninstall_parser.set_defaults(func=uninstall)

    parsed_args = parser.parse_args()
    parsed_args.func(parsed_args)


if __name__ == "__main__":
    main()



import json
import os
from unittest import mock

from awscli.botocore.awsrequest import AWSResponse
from awscli.compat import BytesIO
from scripts.performance import BaseBenchmarkSuite


class RawResponse(BytesIO):
    """
    A bytes-like streamable HTTP response representation.
    """

    def stream(self, **kwargs):
        contents = self.read()
        while contents:
            yield contents
            contents = self.read()


class StubbedHTTPClient:
    def _get_response(self, request):
        response = self._responses.pop(0)
        if isinstance(response, Exception):
            raise response
        return response

    def setup(self):
        urllib3_session_send = 'botocore.httpsession.URLLib3Session.send'
        self._urllib3_patch = mock.patch(urllib3_session_send)
        self._send = self._urllib3_patch.start()
        self._send.side_effect = self._get_response
        self._responses = []

    def tear_down(self):
        self._urllib3_patch.stop()

    def add_response(self, body, headers, status_code):
        response = AWSResponse(
            url='http://169.254.169.254/',
            status_code=status_code,
            headers=headers,
            raw=RawResponse(body.encode()),
        )
        self._responses.append(response)


class JSONStubbedBenchmarkSuite(BaseBenchmarkSuite):
    def __init__(self):
        self._client = StubbedHTTPClient()
        self._benchmark_results = {}

    def _create_file_with_size(self, path, size):
        """
        Creates a full-access file in the given directory with the
        specified name and size. The created file will be full of
        null bytes to achieve the specified size.
        """
        with open(path, 'wb') as f:
            os.chmod(path, 0o777)
            size = int(size)
            f.truncate(size)

    def _create_file_dir(self, dir_path, file_count, size):
        """
        Creates a directory with the specified name. Also creates identical files
        with the given size in the created directory. The number of identical files
        to be created is specified by file_count. Each file will be full of
        null bytes to achieve the specified size.
        """
        os.mkdir(dir_path, 0o777)
        for i in range(int(file_count)):
            file_path = os.path.join(dir_path, f'{i}')
            self._create_file_with_size(file_path, size)

    def _stub_responses(self, responses, client: StubbedHTTPClient):
        """
        Stubs the supplied HTTP client using the response instructions in the supplied
        responses struct. Each instruction will generate one or more stubbed responses.
        """
        for response in responses:
            body = response.get("body", "")
            headers = response.get("headers", {})
            status_code = response.get("status_code", 200)
            # use the instances key to support duplicating responses a configured number of times
            if "instances" in response:
                for _ in range(int(response['instances'])):
                    client.add_response(body, headers, status_code)
            else:
                client.add_response(body, headers, status_code)

    def _get_env_vars(self, config_path):
        return {
            'AWS_CONFIG_FILE': config_path,
            'AWS_DEFAULT_REGION': 'us-west-2',
        }

    def get_test_cases(self, args):
        definitions = json.load(open(args.benchmark_definitions))

        def generator(definition):
            for iteration in range(args.num_iterations):
                yield definition

        return [generator(definition) for definition in definitions]

    def begin_iteration(self, case, workspace_path, assets_path, iteration):
        env = case.get('environment', {})
        config_path = os.path.join(assets_path, 'config')
        self._client.setup()
        self._stub_responses(case.get('responses', []), self._client)
        os.makedirs(os.path.dirname(config_path), mode=0o777, exist_ok=True)
        with open(config_path, 'w') as f:
            f.write(env.get('config', "[DEFAULT]"))
            f.flush()
        self._env_patch = mock.patch.dict(
            'os.environ', self._get_env_vars(config_path)
        )
        self._env_patch.start()
        if "files" in env:
            for file_def in env['files']:
                path = os.path.join(workspace_path, file_def['name'])
                self._create_file_with_size(path, file_def['size'])
        if "file_dirs" in env:
            for file_dir_def in env['file_dirs']:
                dir_path = os.path.join(workspace_path, file_dir_def['name'])
                self._create_file_dir(
                    dir_path,
                    file_dir_def['file_count'],
                    file_dir_def['file_size'],
                )
        if "file_literals" in env:
            for file_lit in env['file_literals']:
                path = os.path.join(workspace_path, file_lit['name'])
                with open(path, file_lit.get('mode', 'w')) as f:
                    os.chmod(path, 0o777)
                    f.write(file_lit['content'])

    def end_iteration(self, case, iteration):
        self._client.tear_down()
        self._env_patch.stop()



# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import threading
import time
from concurrent.futures import CancelledError
from io import BytesIO

from botocore.config import Config
from s3transfer.manager import TransferConfig

from tests import (
    NonSeekableReader,
    RecordingSubscriber,
    skip_if_using_serial_implementation,
)
from tests.integration.s3transfer import (
    BaseTransferManagerIntegTest,
    WaitForTransferStart,
)


class TestUpload(BaseTransferManagerIntegTest):
    def setUp(self):
        super().setUp()
        self.multipart_threshold = 5 * 1024 * 1024
        self.config = TransferConfig(
            multipart_threshold=self.multipart_threshold
        )
        # Client for ensuring any exceptions we throw while sending the body
        # are not retried for the purposes of testing the ability to exit
        # quickly.
        self.client_with_no_retries = self.create_client(
            config=Config(retries={'max_attempts': 1})
        )

    def get_input_fileobj(self, size, name=''):
        return self.files.create_file_with_size(name, size)

    def test_upload_below_threshold(self):
        transfer_manager = self.create_transfer_manager(self.config)
        file = self.get_input_fileobj(size=1024 * 1024, name='1mb.txt')
        future = transfer_manager.upload(file, self.bucket_name, '1mb.txt')
        self.addCleanup(self.delete_object, '1mb.txt')

        future.result()
        self.assertTrue(self.object_exists('1mb.txt'))

    def test_upload_above_threshold(self):
        transfer_manager = self.create_transfer_manager(self.config)
        file = self.get_input_fileobj(size=20 * 1024 * 1024, name='20mb.txt')
        future = transfer_manager.upload(file, self.bucket_name, '20mb.txt')
        self.addCleanup(self.delete_object, '20mb.txt')

        future.result()
        self.assertTrue(self.object_exists('20mb.txt'))

    @skip_if_using_serial_implementation(
        'Exception is thrown once the transfer is submitted. '
        'However for the serial implementation, transfers are performed '
        'in main thread meaning the transfer will complete before the '
        'KeyboardInterrupt being thrown.'
    )
    def test_large_upload_exits_quicky_on_exception(self):
        transfer_manager = self.create_transfer_manager(
            self.config, self.client_with_no_retries
        )

        filename = self.get_input_fileobj(
            name='foo.txt', size=20 * 1024 * 1024
        )

        timeout = 10
        bytes_transferring = threading.Event()
        subscriber = WaitForTransferStart(bytes_transferring)
        try:
            with transfer_manager:
                future = transfer_manager.upload(
                    filename,
                    self.bucket_name,
                    '20mb.txt',
                    subscribers=[subscriber],
                )
                if not bytes_transferring.wait(timeout):
                    future.cancel()
                    raise RuntimeError(
                        "Download transfer did not start after waiting for "
                        f"{timeout} seconds."
                    )
                # Raise an exception which should cause the preceding
                # download to cancel and exit quickly
                start_time = time.time()
                raise KeyboardInterrupt()
        except KeyboardInterrupt:
            pass
        end_time = time.time()
        # The maximum time allowed for the transfer manager to exit.
        # This means that it should take less than a couple second after
        # sleeping to exit.
        max_allowed_exit_time = 5
        actual_time_to_exit = end_time - start_time
        self.assertLess(
            actual_time_to_exit,
            max_allowed_exit_time,
            f"Failed to exit under {max_allowed_exit_time}. Instead exited in {actual_time_to_exit}.",
        )

        try:
            future.result()
            self.skipTest(
                'Upload completed before interrupted and therefore '
                'could not cancel the upload'
            )
        except CancelledError as e:
            self.assertEqual(str(e), 'KeyboardInterrupt()')
            # If the transfer did get cancelled,
            # make sure the object does not exist.
            self.assertTrue(self.object_not_exists('20mb.txt'))

    @skip_if_using_serial_implementation(
        'Exception is thrown once the transfers are submitted. '
        'However for the serial implementation, transfers are performed '
        'in main thread meaning the transfers will complete before the '
        'KeyboardInterrupt being thrown.'
    )
    def test_many_files_exits_quicky_on_exception(self):
        # Set the max request queue size and number of submission threads
        # to something small to simulate having a large queue
        # of transfer requests to complete and it is backed up.
        self.config.max_request_queue_size = 1
        self.config.max_submission_concurrency = 1
        transfer_manager = self.create_transfer_manager(
            self.config, self.client_with_no_retries
        )

        fileobjs = []
        keynames = []
        futures = []
        for i in range(10):
            filename = 'file' + str(i)
            keynames.append(filename)
            fileobjs.append(
                self.get_input_fileobj(name=filename, size=1024 * 1024)
            )

        try:
            with transfer_manager:
                for i, fileobj in enumerate(fileobjs):
                    futures.append(
                        transfer_manager.upload(
                            fileobj, self.bucket_name, keynames[i]
                        )
                    )
                # Raise an exception which should cause the preceding
                # transfer to cancel and exit quickly
                start_time = time.time()
                raise KeyboardInterrupt()
        except KeyboardInterrupt:
            pass
        end_time = time.time()
        # The maximum time allowed for the transfer manager to exit.
        # This means that it should take less than a couple seconds to exit.
        max_allowed_exit_time = 5
        self.assertLess(
            end_time - start_time,
            max_allowed_exit_time,
            f"Failed to exit under {max_allowed_exit_time}. Instead exited in {end_time - start_time}.",
        )

        # Make sure at least one of the futures got cancelled
        with self.assertRaisesRegex(CancelledError, 'KeyboardInterrupt()'):
            for future in futures:
                future.result()
        # For the transfer that did get cancelled, make sure the object
        # does not exist.
        self.assertTrue(self.object_not_exists(future.meta.call_args.key))

    def test_progress_subscribers_on_upload(self):
        subscriber = RecordingSubscriber()
        transfer_manager = self.create_transfer_manager(self.config)
        file = self.get_input_fileobj(size=20 * 1024 * 1024, name='20mb.txt')
        future = transfer_manager.upload(
            file, self.bucket_name, '20mb.txt', subscribers=[subscriber]
        )
        self.addCleanup(self.delete_object, '20mb.txt')

        future.result()
        # The callback should have been called enough times such that
        # the total amount of bytes we've seen (via the "amount"
        # arg to the callback function) should be the size
        # of the file we uploaded.
        self.assertEqual(subscriber.calculate_bytes_seen(), 20 * 1024 * 1024)


class TestUploadSeekableStream(TestUpload):
    def get_input_fileobj(self, size, name=''):
        return BytesIO(b'0' * size)


class TestUploadNonSeekableStream(TestUpload):
    def get_input_fileobj(self, size, name=''):
        return NonSeekableReader(b'0' * size)



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0e
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import argparse
import os
import sys

import botocore.session

from awscli.compat import StringIO
from awscli.customizations.exceptions import ParamValidationError
from awscli.customizations.s3.s3 import S3
from awscli.customizations.s3.subcommands import (
    CommandArchitecture,
    CommandParameters,
    CpCommand,
    ListCommand,
    RbCommand,
    SyncCommand,
)
from awscli.customizations.s3.syncstrategy.base import (
    MissingFileSync,
    NeverSync,
    SizeAndLastModifiedSync,
)
from awscli.customizations.s3.transferconfig import RuntimeConfig
from awscli.testutils import (
    BaseAWSCommandParamsTest,
    BaseAWSHelpOutputTest,
    FileCreator,
    mock,
    unittest,
)
from tests.unit.customizations.s3 import clean_loc_files, make_loc_files


class FakeArgs:
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    def __contains__(self, key):
        return key in self.__dict__


class TestRbCommand(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock()
        self.session.get_scoped_config.return_value = {}
        self.rb_command = RbCommand(self.session)
        self.parsed_args = FakeArgs(
            path='s3://mybucket/', force=True, dir_op=False
        )
        self.parsed_globals = FakeArgs(
            region=None, endpoint_url=None, verify_ssl=None
        )
        self.cmd_name = 'awscli.customizations.s3.subcommands.RmCommand'
        self.arch_name = (
            'awscli.customizations.s3.subcommands.CommandArchitecture'
        )

    def test_rb_command_with_force_deletes_objects_in_bucket(self):
        with mock.patch(self.cmd_name) as rm_command:
            with mock.patch(self.arch_name):
                # RmCommand returns an RmCommand instance whose __call__
                # should be the RC of the command.
                # In this case we'll have it return an RC of 0 which indicates
                # success.
                rm_command.return_value.return_value = 0
                self.rb_command._run_main(
                    self.parsed_args, parsed_globals=self.parsed_globals
                )
            # Because of --force we should have called the
            # rm_command with the --recursive option.
            rm_command.return_value.assert_called_with(
                ['s3://mybucket/', '--recursive'], mock.ANY
            )

    def test_rb_command_with_force_requires_strict_path(self):
        with self.assertRaises(ParamValidationError):
            self.parsed_args.path = 's3://mybucket/mykey'
            self.rb_command._run_main(
                self.parsed_args, parsed_globals=self.parsed_globals
            )


class TestLSCommand(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock()
        self.session.create_client.return_value.list_buckets.return_value = {
            'Buckets': []
        }
        self.session.create_client.return_value.get_paginator.return_value.paginate.return_value = [
            {'Contents': [], 'CommonPrefixes': []}
        ]

    def _get_fake_kwargs(self, override=None):
        fake_kwargs = {
            'paths': 's3://',
            'dir_op': False,
            'human_readable': False,
            'summarize': False,
            'page_size': None,
            'request_payer': None,
            'bucket_name_prefix': None,
            'bucket_region': None,
        }
        fake_kwargs.update(override or {})

        return fake_kwargs

    def test_ls_command_for_bucket(self):
        ls_command = ListCommand(self.session)
        parsed_args = FakeArgs(
            **self._get_fake_kwargs(
                {
                    'paths': 's3://mybucket/',
                    'page_size': '5',
                }
            )
        )
        parsed_globals = mock.Mock()
        ls_command._run_main(parsed_args, parsed_globals)
        call = self.session.create_client.return_value.list_objects_v2
        paginate = self.session.create_client.return_value.get_paginator.return_value.paginate
        # We should make no operation calls.
        self.assertEqual(call.call_count, 0)
        # And only a single pagination call to ListObjectsV2.
        self.session.create_client.return_value.get_paginator.assert_called_with(
            'list_objects_v2'
        )
        ref_call_args = {
            'Bucket': 'mybucket',
            'Delimiter': '/',
            'Prefix': '',
            'PaginationConfig': {'PageSize': '5'},
        }

        paginate.assert_called_with(**ref_call_args)

    def test_ls_command_with_no_args(self):
        ls_command = ListCommand(self.session)
        parsed_global = FakeArgs(
            region=None, endpoint_url=None, verify_ssl=None
        )
        parsed_args = FakeArgs(**self._get_fake_kwargs())
        ls_command._run_main(parsed_args, parsed_global)
        call = self.session.create_client.return_value.list_buckets
        paginate = self.session.create_client.return_value.get_paginator.return_value.paginate

        # We should make no operation calls.
        self.assertEqual(call.call_count, 0)
        # And only a single pagination call to ListBuckets.
        self.session.create_client.return_value.get_paginator.assert_called_with(
            'list_buckets'
        )
        ref_call_args = {'PaginationConfig': {'PageSize': None}}

        paginate.assert_called_with(**ref_call_args)

        # Verify get_client
        get_client = self.session.create_client
        args = get_client.call_args
        self.assertEqual(
            args,
            mock.call('s3', region_name=None, verify=None, endpoint_url=None),
        )

    def test_ls_with_bucket_name_prefix(self):
        ls_command = ListCommand(self.session)
        parsed_args = FakeArgs(
            **self._get_fake_kwargs(
                {
                    'bucket_name_prefix': 'myprefix',
                }
            )
        )
        parsed_globals = FakeArgs(
            region=None,
            endpoint_url=None,
            verify_ssl=None,
        )
        ls_command._run_main(parsed_args, parsed_globals)
        call = self.session.create_client.return_value.list_objects
        paginate = self.session.create_client.return_value.get_paginator.return_value.paginate
        # We should make no operation calls.
        self.assertEqual(call.call_count, 0)
        self.session.create_client.return_value.get_paginator.assert_called_with(
            'list_buckets'
        )
        ref_call_args = {
            'PaginationConfig': {'PageSize': None},
            'Prefix': 'myprefix',
        }

        paginate.assert_called_with(**ref_call_args)

    def test_ls_with_bucket_region(self):
        ls_command = ListCommand(self.session)
        parsed_args = FakeArgs(
            **self._get_fake_kwargs(
                {
                    'bucket_region': 'us-west-1',
                }
            )
        )
        parsed_globals = FakeArgs(
            region=None,
            endpoint_url=None,
            verify_ssl=None,
        )
        ls_command._run_main(parsed_args, parsed_globals)
        call = self.session.create_client.return_value.list_objects
        paginate = self.session.create_client.return_value.get_paginator.return_value.paginate
        # We should make no operation calls.
        self.assertEqual(call.call_count, 0)
        self.session.create_client.return_value.get_paginator.assert_called_with(
            'list_buckets'
        )
        ref_call_args = {
            'PaginationConfig': {'PageSize': None},
            'BucketRegion': 'us-west-1',
        }

        paginate.assert_called_with(**ref_call_args)

    def test_ls_with_verify_argument(self):
        ls_command = ListCommand(self.session)
        parsed_global = FakeArgs(
            region='us-west-2', endpoint_url=None, verify_ssl=False
        )
        parsed_args = FakeArgs(**self._get_fake_kwargs({}))
        ls_command._run_main(parsed_args, parsed_global)
        # Verify get_client
        get_client = self.session.create_client
        args = get_client.call_args
        self.assertEqual(
            args,
            mock.call(
                's3',
                region_name='us-west-2',
                verify=False,
                endpoint_url=None,
            ),
        )

    def test_ls_with_requester_pays(self):
        ls_command = ListCommand(self.session)
        parsed_args = FakeArgs(
            **self._get_fake_kwargs(
                {
                    'paths': 's3://mybucket/',
                    'page_size': '5',
                    'request_payer': 'requester',
                }
            )
        )
        parsed_globals = mock.Mock()
        ls_command._run_main(parsed_args, parsed_globals)
        call = self.session.create_client.return_value.list_objects
        paginate = self.session.create_client.return_value.get_paginator.return_value.paginate
        # We should make no operation calls.
        self.assertEqual(call.call_count, 0)
        # And only a single pagination call to ListObjectsV2.
        self.session.create_client.return_value.get_paginator.assert_called_with(
            'list_objects_v2'
        )
        ref_call_args = {
            'Bucket': 'mybucket',
            'Delimiter': '/',
            'Prefix': '',
            'PaginationConfig': {'PageSize': '5'},
            'RequestPayer': 'requester',
        }

        paginate.assert_called_with(**ref_call_args)


class CommandArchitectureTest(BaseAWSCommandParamsTest):
    def setUp(self):
        super(CommandArchitectureTest, self).setUp()
        self.session = self.driver.session
        self.bucket = 'mybucket'
        self.transfer_manager = mock.Mock()
        self.source_client = mock.Mock()
        self.transfer_client = mock.Mock()
        self.file_creator = FileCreator()
        self.loc_files = make_loc_files(self.file_creator)
        self.output = StringIO()
        self.err_output = StringIO()
        self.saved_stdout = sys.stdout
        self.saved_stderr = sys.stderr
        sys.stdout = self.output
        sys.stderr = self.err_output

    def tearDown(self):
        self.output.close()
        self.err_output.close()
        sys.stdout = self.saved_stdout
        sys.stderr = self.saved_stderr

        super(CommandArchitectureTest, self).tearDown()
        clean_loc_files(self.file_creator)

    def get_cmd_architecture(self, cmd, params):
        return CommandArchitecture(
            session=self.session,
            cmd=cmd,
            parameters=params,
            transfer_manager=self.transfer_manager,
            source_client=self.source_client,
            transfer_client=self.transfer_client,
        )

    def get_params(self, **override_kwargs):
        params = {
            'region': 'us-east-1',
            'endpoint_url': None,
            'verify_ssl': None,
            'is_stream': False,
        }
        params.update(override_kwargs)
        return params

    def test_create_instructions(self):
        """
        This tests to make sure the instructions for any command is generated
        properly.
        """
        cmds = ['cp', 'mv', 'rm', 'sync']

        instructions = {
            'cp': ['file_generator', 'file_info_builder', 's3_handler'],
            'mv': ['file_generator', 'file_info_builder', 's3_handler'],
            'rm': ['file_generator', 'file_info_builder', 's3_handler'],
            'sync': [
                'file_generator',
                'comparator',
                'file_info_builder',
                's3_handler',
            ],
        }

        params = {
            'filters': True,
            'region': 'us-east-1',
            'endpoint_url': None,
            'verify_ssl': None,
            'is_stream': False,
        }
        for cmd in cmds:
            cmd_arc = self.get_cmd_architecture(cmd, self.get_params())
            cmd_arc.create_instructions()
            self.assertEqual(cmd_arc.instructions, instructions[cmd])

        # Test if there is a filter.
        cmd_arc = self.get_cmd_architecture('cp', params)
        cmd_arc.create_instructions()
        self.assertEqual(
            cmd_arc.instructions,
            ['file_generator', 'filters', 'file_info_builder', 's3_handler'],
        )

    def test_choose_sync_strategy_default(self):
        self.session = mock.Mock(self.session)
        cmd_arc = self.get_cmd_architecture('sync', self.get_params())
        # Check if no plugins return their sync strategy.  Should
        # result in the default strategies
        self.session.emit.return_value = None
        sync_strategies = cmd_arc.choose_sync_strategies()
        self.assertEqual(
            sync_strategies['file_at_src_and_dest_sync_strategy'].__class__,
            SizeAndLastModifiedSync,
        )
        self.assertEqual(
            sync_strategies['file_not_at_dest_sync_strategy'].__class__,
            MissingFileSync,
        )
        self.assertEqual(
            sync_strategies['file_not_at_src_sync_strategy'].__class__,
            NeverSync,
        )

    def test_choose_sync_strategy_overwrite(self):
        self.session = mock.Mock(self.session)
        cmd_arc = self.get_cmd_architecture('sync', self.get_params())
        # Check that the default sync strategy is overwritted if a plugin
        # returns its sync strategy.
        mock_strategy = mock.Mock()
        mock_strategy.sync_type = 'file_at_src_and_dest'

        mock_not_at_dest_sync_strategy = mock.Mock()
        mock_not_at_dest_sync_strategy.sync_type = 'file_not_at_dest'

        mock_not_at_src_sync_strategy = mock.Mock()
        mock_not_at_src_sync_strategy.sync_type = 'file_not_at_src'

        responses = [
            (None, mock_strategy),
            (None, mock_not_at_dest_sync_strategy),
            (None, mock_not_at_src_sync_strategy),
        ]

        self.session.emit.return_value = responses
        sync_strategies = cmd_arc.choose_sync_strategies()
        self.assertEqual(
            sync_strategies['file_at_src_and_dest_sync_strategy'],
            mock_strategy,
        )
        self.assertEqual(
            sync_strategies['file_not_at_dest_sync_strategy'],
            mock_not_at_dest_sync_strategy,
        )
        self.assertEqual(
            sync_strategies['file_not_at_src_sync_strategy'],
            mock_not_at_src_sync_strategy,
        )


class CommandParametersTest(unittest.TestCase):
    def setUp(self):
        self.environ = {}
        self.environ_patch = mock.patch('os.environ', self.environ)
        self.environ_patch.start()
        self.mock = mock.MagicMock()
        self.mock.get_config = mock.MagicMock(return_value={'region': None})
        self.file_creator = FileCreator()
        self.loc_files = make_loc_files(self.file_creator)
        self.bucket = 's3testbucket'
        self.session = mock.Mock()
        self.parsed_global = FakeArgs(
            region='us-west-2', endpoint_url=None, verify_ssl=False
        )

    def tearDown(self):
        self.environ_patch.stop()
        clean_loc_files(self.file_creator)

    def test_check_path_type_pass(self):
        # This tests the class's ability to determine whether the correct
        # path types have been passed for a particular command.  It test every
        # possible combination that is correct for every command.
        cmds = {
            'cp': ['locals3', 's3s3', 's3local'],
            'mv': ['locals3', 's3s3', 's3local'],
            'rm': ['s3'],
            'mb': ['s3'],
            'rb': ['s3'],
            'sync': ['locals3', 's3s3', 's3local'],
        }
        s3_file = 's3://' + self.bucket + '/' + 'text1.txt'
        local_file = self.loc_files[0]

        combos = {
            's3s3': [s3_file, s3_file],
            's3local': [s3_file, local_file],
            'locals3': [local_file, s3_file],
            's3': [s3_file],
            'local': [local_file],
            'locallocal': [local_file, local_file],
        }

        for cmd in cmds.keys():
            cmd_param = CommandParameters(
                cmd, {}, '', self.session, self.parsed_global
            )
            cmd_param.add_region(mock.Mock())
            correct_paths = cmds[cmd]
            for path_args in correct_paths:
                cmd_param.check_path_type(combos[path_args])

    def test_check_path_type_fail(self):
        # This tests the class's ability to determine whether the correct
        # path types have been passed for a particular command. It test every
        # possible combination that is incorrect for every command.
        cmds = {
            'cp': ['local', 'locallocal', 's3'],
            'mv': ['local', 'locallocal', 's3'],
            'rm': ['local', 'locallocal', 's3s3', 'locals3', 's3local'],
            'ls': ['local', 'locallocal', 's3s3', 'locals3', 's3local'],
            'sync': ['local', 'locallocal', 's3'],
            'mb': ['local', 'locallocal', 's3s3', 'locals3', 's3local'],
            'rb': ['local', 'locallocal', 's3s3', 'locals3', 's3local'],
        }
        s3_file = 's3://' + self.bucket + '/' + 'text1.txt'
        local_file = self.loc_files[0]

        combos = {
            's3s3': [s3_file, s3_file],
            's3local': [s3_file, local_file],
            'locals3': [local_file, s3_file],
            's3': [s3_file],
            'local': [local_file],
            'locallocal': [local_file, local_file],
        }

        for cmd in cmds.keys():
            cmd_param = CommandParameters(
                cmd, {}, '', self.session, self.parsed_global
            )
            cmd_param.add_region(mock.Mock())
            wrong_paths = cmds[cmd]
            for path_args in wrong_paths:
                with self.assertRaises(ParamValidationError):
                    cmd_param.check_path_type(combos[path_args])

    def test_validate_streaming_paths_upload(self):
        paths = ['-', 's3://bucket']
        cmd_params = CommandParameters('cp', {}, '')
        cmd_params.add_paths(paths)
        self.assertTrue(cmd_params.parameters['is_stream'])
        self.assertTrue(cmd_params.parameters['only_show_errors'])
        self.assertFalse(cmd_params.parameters['dir_op'])

    def test_validate_streaming_paths_download(self):
        paths = ['s3://bucket/key', '-']
        cmd_params = CommandParameters('cp', {}, '')
        cmd_params.add_paths(paths)
        self.assertTrue(cmd_params.parameters['is_stream'])
        self.assertTrue(cmd_params.parameters['only_show_errors'])
        self.assertFalse(cmd_params.parameters['dir_op'])

    def test_validate_streaming_paths_with_no_overwrite(self):
        paths = ['s3://bucket/key', '-']
        cmd_params = CommandParameters('cp', {'no_overwrite': True}, '')
        with self.assertRaises(ParamValidationError) as cm:
            cmd_params.add_paths(paths)
            self.assertIn(
                '--no-overwrite parameter is not supported for streaming downloads',
                cm.msg,
            )

    def test_validate_no_streaming_paths(self):
        paths = [self.file_creator.rootdir, 's3://bucket']
        cmd_params = CommandParameters('cp', {}, '')
        cmd_params.add_paths(paths)
        self.assertFalse(cmd_params.parameters['is_stream'])

    def test_validate_checksum_algorithm_download_error(self):
        paths = ['s3://bucket/key', self.file_creator.rootdir]
        parameters = {'checksum_algorithm': 'CRC32'}
        cmd_params = CommandParameters('cp', parameters, '')
        with self.assertRaises(ParamValidationError) as cm:
            cmd_params.add_paths(paths)
            self.assertIn(
                'Expected checksum-algorithm parameter to be used with one of following path formats',
                cm.msg,
            )

    def test_validate_checksum_algorithm_sync_download_error(self):
        paths = ['s3://bucket/key', self.file_creator.rootdir]
        parameters = {'checksum_algorithm': 'CRC32C'}
        cmd_params = CommandParameters('sync', parameters, '')
        with self.assertRaises(ParamValidationError) as cm:
            cmd_params.add_paths(paths)
            self.assertIn(
                'Expected checksum-algorithm parameter to be used with one of following path formats',
                cm.msg,
            )

    def test_validate_checksum_mode_upload_error(self):
        paths = [self.file_creator.rootdir, 's3://bucket/key']
        parameters = {'checksum_mode': 'ENABLED'}
        cmd_params = CommandParameters('cp', parameters, '')
        with self.assertRaises(ParamValidationError) as cm:
            cmd_params.add_paths(paths)
            self.assertIn(
                'Expected checksum-mode parameter to be used with one of following path formats',
                cm.msg,
            )

    def test_validate_checksum_mode_sync_upload_error(self):
        paths = [self.file_creator.rootdir, 's3://bucket/key']
        parameters = {'checksum_mode': 'ENABLED'}
        cmd_params = CommandParameters('sync', parameters, '')
        with self.assertRaises(ParamValidationError) as cm:
            cmd_params.add_paths(paths)
            self.assertIn(
                'Expected checksum-mode parameter to be used with one of following path formats',
                cm.msg,
            )

    def test_validate_checksum_mode_move_error(self):
        paths = ['s3://bucket/key', 's3://bucket2/key']
        parameters = {'checksum_mode': 'ENABLED'}
        cmd_params = CommandParameters('mv', parameters, '')
        with self.assertRaises(ParamValidationError) as cm:
            cmd_params.add_paths(paths)
            self.assertIn(
                'Expected checksum-mode parameter to be used with one of following path formats',
                cm.msg,
            )

    def test_validate_streaming_paths_error(self):
        parameters = {'src': '-', 'dest': 's3://bucket'}
        cmd_params = CommandParameters('sync', parameters, '')
        with self.assertRaises(ParamValidationError):
            cmd_params._validate_streaming_paths()

    def test_validate_non_existent_local_path_upload(self):
        non_existent_path = os.path.join(self.file_creator.rootdir, 'foo')
        paths = [non_existent_path, 's3://bucket/']
        cmd_param = CommandParameters('cp', {}, '')
        with self.assertRaises(RuntimeError):
            cmd_param.add_paths(paths)

    def test_add_path_for_non_existsent_local_path_download(self):
        non_existent_path = os.path.join(self.file_creator.rootdir, 'foo')
        paths = ['s3://bucket', non_existent_path]
        cmd_param = CommandParameters('cp', {'dir_op': True}, '')
        cmd_param.add_paths(paths)
        self.assertTrue(os.path.exists(non_existent_path))

    def test_validate_sse_c_args_missing_sse(self):
        paths = ['s3://bucket/foo', 's3://bucket/bar']
        params = {'dir_op': False, 'sse_c_key': 'foo'}
        cmd_param = CommandParameters('cp', params, '')
        error_msg = '--sse-c must be specified'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            cmd_param.add_paths(paths)

    def test_validate_sse_c_args_missing_sse_c_key(self):
        paths = ['s3://bucket/foo', 's3://bucket/bar']
        params = {'dir_op': False, 'sse_c': 'AES256'}
        cmd_param = CommandParameters('cp', params, '')
        error_msg = '--sse-c-key must be specified'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            cmd_param.add_paths(paths)

    def test_validate_sse_c_args_missing_sse_c_copy_source(self):
        paths = ['s3://bucket/foo', 's3://bucket/bar']
        params = {'dir_op': False, 'sse_c_copy_source_key': 'foo'}
        cmd_param = CommandParameters('cp', params, '')
        error_msg = '--sse-c-copy-source must be specified'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            cmd_param.add_paths(paths)

    def test_validate_sse_c_args_missing_sse_c_copy_source_key(self):
        paths = ['s3://bucket/foo', 's3://bucket/bar']
        params = {'dir_op': False, 'sse_c_copy_source': 'AES256'}
        cmd_param = CommandParameters('cp', params, '')
        error_msg = '--sse-c-copy-source-key must be specified'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            cmd_param.add_paths(paths)

    def test_validate_sse_c_args_wrong_path_type(self):
        paths = ['s3://bucket/foo', self.file_creator.rootdir]
        params = {
            'dir_op': False,
            'sse_c_copy_source': 'AES256',
            'sse_c_copy_source_key': 'foo',
        }
        cmd_param = CommandParameters('cp', params, '')
        error_msg = 'only supported for copy operations'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            cmd_param.add_paths(paths)

    def test_adds_is_move(self):
        params = {}
        CommandParameters(
            'mv',
            params,
            '',
            session=self.session,
            parsed_globals=self.parsed_global,
        )
        self.assertTrue(params.get('is_move'))

        # is_move should only be true for mv
        params = {}
        CommandParameters('cp', params, '')
        self.assertFalse(params.get('is_move'))


class HelpDocTest(BaseAWSHelpOutputTest):
    def setUp(self):
        super(HelpDocTest, self).setUp()
        self.session = botocore.session.get_session()

    def tearDown(self):
        super(HelpDocTest, self).tearDown()

    def test_s3_help(self):
        # This tests the help command for the s3 service. This
        # checks to make sure the appropriate descriptions are
        # added including the tutorial.
        s3 = S3(self.session)
        parser = argparse.ArgumentParser()
        parser.add_argument('--paginate', action='store_true')
        parsed_global = parser.parse_args(['--paginate'])
        help_command = s3.create_help_command()
        help_command([], parsed_global)
        self.assert_contains(
            "This section explains prominent concepts "
            "and notations in the set of high-level S3 commands provided."
        )
        self.assert_contains("Every command takes one or two positional")
        self.assert_contains("* rb")

    def test_s3command_help(self):
        # This tests the help command for an s3 command. This
        # checks to make sure the command prints appropriate
        # parts.  Note the examples are not included because
        # the event was not registered.
        s3command = CpCommand(self.session)
        s3command._arg_table = s3command._build_arg_table()
        parser = argparse.ArgumentParser()
        parser.add_argument('--paginate', action='store_true')
        parsed_global = parser.parse_args(['--paginate'])
        help_command = s3command.create_help_command()
        help_command([], parsed_global)
        self.assert_contains("cp")
        self.assert_contains("[--acl <value>]")
        self.assert_contains("Displays the operations that would be")

    def test_help(self):
        # This ensures that the file appropriately redirects to help object
        # if help is the only argument left to be parsed.  There should not
        # have any contents in the docs.
        s3_command = SyncCommand(self.session)
        s3_command(['help'], [])
        self.assert_contains('sync')
        self.assert_contains("Synopsis")


if __name__ == "__main__":
    unittest.main()



# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import aws, unittest


class TestDescribeInstances(unittest.TestCase):
    def setUp(self):
        self.prefix = 'ec2 describe-instances --region us-west-2'

    def test_describe_instances_with_id(self):
        command = self.prefix + ' --instance-ids malformed-id'
        result = aws(command)
        self.assertIn('InvalidInstanceID.Malformed', result.stderr)

    def test_describe_instances_with_filter(self):
        command = self.prefix + ' --filters Name=instance-id,Values=['
        command += 'malformed-id]'
        result = aws(command)
        reservations = result.json["Reservations"]
        self.assertEqual(len(reservations), 0)


class TestDescribeSnapshots(unittest.TestCase):
    def setUp(self):
        self.prefix = 'ec2 describe-snapshots --region us-west-2'

    def test_describe_snapshot_with_snapshot_id(self):
        command = self.prefix + ' --snapshot-ids malformed-id'
        result = aws(command)
        self.assertIn('InvalidParameterValue', result.stderr)

    def test_describe_snapshots_with_filter(self):
        command = self.prefix
        command += ' --filters Name=snapshot-id,Values=[malformed-id]'
        result = aws(command)
        snapshots = result.json['Snapshots']
        self.assertEqual(len(snapshots), 0)


class TestDescribeVolumes(unittest.TestCase):
    def setUp(self):
        self.prefix = 'ec2 describe-volumes --region us-west-2'

    def test_describe_volumes_with_volume_id(self):
        command = self.prefix + ' --volume-ids malformed-id'
        result = aws(command)
        self.assertIn('InvalidParameterValue', result.stderr)

    def test_describe_volumes_with_filter(self):
        command = self.prefix
        command += ' --filters Name=volume-id,Values=malformed-id'
        result = aws(command)
        volumes = result.json['Volumes']
        self.assertEqual(len(volumes), 0)



# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import json
import os

from awscli.testutils import mock
from tests.functional.sso import BaseSSOTest


class TestLogoutCommand(BaseSSOTest):
    def setUp(self):
        super(TestLogoutCommand, self).setUp()
        self.logout_token_cache_dir_patch = mock.patch(
            'awscli.customizations.sso.logout.SSO_TOKEN_DIR',
            self.token_cache_dir,
        )
        self.logout_token_cache_dir_patch.start()
        self.aws_creds_cache_dir = self.files.full_path('aws-creds-cache')
        self.aws_creds_cache_dir_patch = mock.patch(
            'awscli.customizations.sso.logout.AWS_CREDS_CACHE_DIR',
            self.aws_creds_cache_dir,
        )
        self.aws_creds_cache_dir_patch.start()

    def tearDown(self):
        super(TestLogoutCommand, self).tearDown()
        self.logout_token_cache_dir_patch.stop()
        self.aws_creds_cache_dir_patch.stop()

    def add_cached_token(self, filename):
        token_path = os.path.join(self.token_cache_dir, filename)
        token_contents = {
            'region': self.sso_region,
            'startUrl': self.start_url,
            'expiresAt': '2019-10-26T05:19:09UTC',
            'accessToken': self.access_token,
        }
        self.files.create_file(token_path, json.dumps(token_contents))
        return token_path

    def add_cached_aws_credentials(self, filename, from_sso=True):
        creds_contents = {
            'Credentials': {
                'AccessKeyId': 'access-key',
                'SecretAccessKey': 'secret-key',
                "SessionToken": 'session-token',
                "Expiration": '2020-01-23T20:48:59UTC',
            },
        }
        if from_sso:
            creds_contents['ProviderType'] = 'sso'
        creds_path = os.path.join(self.aws_creds_cache_dir, filename)
        self.files.create_file(creds_path, json.dumps(creds_contents))
        return creds_path

    def assert_file_exists(self, filename):
        self.assertTrue(os.path.exists(filename))

    def assert_file_does_not_exist(self, filename):
        self.assertFalse(os.path.exists(filename))

    def test_logout(self):
        token = self.add_cached_token('token.json')
        creds = self.add_cached_aws_credentials('sso-creds.json')
        self.run_cmd('sso logout')
        self.assert_file_does_not_exist(token)
        self.assert_file_does_not_exist(creds)

    def test_logout_multiple_cached_files(self):
        token = self.add_cached_token('token.json')
        token2 = self.add_cached_token('token2.json')
        creds = self.add_cached_aws_credentials('sso-creds.json')
        creds2 = self.add_cached_aws_credentials('sso-creds2.json')
        self.run_cmd('sso logout')
        self.assert_file_does_not_exist(token)
        self.assert_file_does_not_exist(token2)
        self.assert_file_does_not_exist(creds)
        self.assert_file_does_not_exist(creds2)

    def test_logout_ignores_non_sso_tokens(self):
        registration_token = os.path.join(
            self.token_cache_dir, 'botocore-client-id.json'
        )
        self.files.create_file(
            registration_token, json.dumps({'clientId': 'myid'})
        )
        self.run_cmd('sso logout')
        self.assert_file_exists(registration_token)

    def test_logout_ignores_non_sso_retrieved_aws_creds(self):
        creds = self.add_cached_aws_credentials('creds.json', from_sso=False)
        self.run_cmd('sso logout')
        self.assert_file_exists(creds)

    def test_ignores_invalid_json_files(self):
        invalid_json = os.path.join(self.token_cache_dir, 'invalid.json')
        self.files.create_file(invalid_json, '{not-json')
        self.run_cmd('sso logout')
        self.assert_file_exists(invalid_json)

    def test_does_not_fail_when_cache_does_not_exist(self):
        self.assertFalse(os.path.exists(self.token_cache_dir))
        self.assertFalse(os.path.exists(self.aws_creds_cache_dir))
        self.run_cmd('sso logout', expected_rc=0)

    def test_calls_sso_logout_with_token(self):
        token = self.add_cached_token('token.json')
        self.run_cmd('sso logout')
        self.assert_file_does_not_exist(token)
        self.assertEqual(len(self.operations_called), 1)
        self.assertEqual(self.operations_called[0][0].name, 'Logout')
        expected_logout_params = {
            'accessToken': self.access_token,
        }
        self.assertEqual(self.operations_called[0][1], expected_logout_params)

    def test_calls_sso_logout_and_handles_client_error(self):
        self.http_response.status_code = 400
        token = self.add_cached_token('token.json')
        self.run_cmd('sso logout', expected_rc=0)
        self.assert_file_does_not_exist(token)

    @mock.patch('botocore.session.Session.create_client')
    def test_logout_with_custom_ca(self, create_client_patch):
        token = self.add_cached_token('token.json')
        creds = self.add_cached_aws_credentials('sso-creds.json')
        self.run_cmd('sso logout --ca-bundle /path/to/ca/bundle.pem')
        create_client_patch.assert_called_once_with(
            'sso',
            region_name='us-west-2',
            verify='/path/to/ca/bundle.pem',
        )
        self.assert_file_does_not_exist(token)
        self.assert_file_does_not_exist(creds)

    @mock.patch('botocore.session.Session.create_client')
    def test_logout_no_verify_ssl(self, create_client_patch):
        token = self.add_cached_token('token.json')
        creds = self.add_cached_aws_credentials('sso-creds.json')
        self.run_cmd('sso logout --no-verify-ssl')
        create_client_patch.assert_called_once_with(
            'sso',
            region_name='us-west-2',
            verify=False,
        )
        self.assert_file_does_not_exist(token)
        self.assert_file_does_not_exist(creds)



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import json

from awscli.customizations.ecs.deploy import (
    MAX_WAIT_MIN,
    TIMEOUT_BUFFER_MIN,
    CodeDeployer,
)
from awscli.customizations.ecs.filehelpers import (
    get_app_name,
    get_deploy_group_name,
)
from awscli.testutils import BaseAWSCommandParamsTest, FileCreator


class TestDeployCommand(BaseAWSCommandParamsTest):
    PREFIX = 'ecs deploy '

    TASK_DEFINITION_JSON = {
        "family": "test",
        "containerDefinitions": [],
        "cpu": "256",
        "memory": '512',
    }

    JSON_APPSPEC = """
    {
        "version": 0.0,
        "resources": [{
            "TestService": {
                "type": "AWS::ECS::Service",
                "properties": {
                    "taskDefinition": "arn:aws:ecs::123:task-definition:1",
                    "loadBalancerInfo": {
                        "containerName": "web",
                        "containerPort": 80
                    }
                }
            }
        }]
    }
    """

    BAD_APPSPEC = """
    version: 0.0
    resources:
      - TestService:
          type: AWS::Lambda::Function
          properties:
            name: myFunction
    """

    YAML_APPSPEC = """
    version: 0.0
    resources:
      - TestService:
          type: AWS::ECS::Service
          properties:
            taskDefinition: arn:aws:ecs::123:task-definition:1
            loadBalancerInfo:
              containerName: web
              containerPort: 80
    """

    APPSPEC_DICT = {
        "version": 0.0,
        "resources": [
            {
                "TestService": {
                    "type": "AWS::ECS::Service",
                    "properties": {
                        "taskDefinition": "arn:aws:ecs::123:task-definition:1",
                        "loadBalancerInfo": {
                            "containerName": "web",
                            "containerPort": 80,
                        },
                    },
                }
            }
        ],
    }

    def setUp(self):
        super(TestDeployCommand, self).setUp()
        # setup required values
        files = FileCreator()
        self.task_def_file = files.create_file(
            'taskDef.json', json.dumps(self.TASK_DEFINITION_JSON), mode='w'
        )
        self.appspec_file = files.create_file(
            'appspec.yaml', self.YAML_APPSPEC, mode='w'
        )
        self.appspec_file_json = files.create_file(
            'appspec.json', self.JSON_APPSPEC, mode='w'
        )
        self.service_name = 'serviceTest'
        self.service_arn = 'arn:aws:ecs:::service/serviceTest'
        # setup default optional values
        self.cluster_name = 'default'
        self.cluster_arn = 'arn:aws:ecs:::cluster/default'
        self.application_name = get_app_name(
            self.service_name, self.cluster_name, None
        )
        self.deployment_group_name = get_deploy_group_name(
            self.service_name, self.cluster_name, None
        )
        # setup test response resources
        self.missing_properties_appspec = files.create_file(
            'appspec_bad.yaml', self.BAD_APPSPEC, mode='w'
        )
        self.task_definition_arn = (
            'arn:aws:ecs::1234567890:task-definition\\test:2'
        )
        self.deployment_id = 'd-1234567XX'
        self.mock_deployer = CodeDeployer(None, self.APPSPEC_DICT)
        self.mock_deployer.update_task_def_arn(self.task_definition_arn)
        self.expected_stdout = (
            "Successfully registered new ECS task "
            "definition " + self.task_definition_arn + "\n"
            "Successfully created deployment " + self.deployment_id + "\n"
            "Waiting for "
            + self.deployment_id
            + " to succeed (will wait up to 30 minutes)..."
            "\nSuccessfully deployed "
            + self.task_definition_arn
            + " to service '"
            + self.service_name
            + "'\n"
        )

    def tearDown(self):
        super(TestDeployCommand, self).tearDown()

    def test_deploy_with_defaults(self):
        cmdline = self.PREFIX
        cmdline += '--service ' + self.service_name
        cmdline += ' --task-definition ' + self.task_def_file
        cmdline += ' --codedeploy-appspec ' + self.appspec_file

        expected_create_deployment_params = (
            self.mock_deployer._get_create_deploy_request(
                self.application_name, self.deployment_group_name
            )
        )

        self.parsed_responses = self._get_parsed_responses(
            self.cluster_name,
            self.application_name,
            self.deployment_group_name,
        )

        expected_params = self._get_expected_params(
            self.service_name,
            self.cluster_name,
            self.application_name,
            self.deployment_group_name,
            expected_create_deployment_params,
        )

        stdout, _, _ = self.assert_params_list_for_cmd(
            cmdline, expected_params, 0
        )

        self.assertEqual(stdout, self.expected_stdout)

    def test_deploy_with_default_arns(self):
        cmdline = self.PREFIX
        cmdline += '--service ' + self.service_arn
        cmdline += ' --cluster ' + self.cluster_arn
        cmdline += ' --task-definition ' + self.task_def_file
        cmdline += ' --codedeploy-appspec ' + self.appspec_file

        expected_create_deployment_params = (
            self.mock_deployer._get_create_deploy_request(
                self.application_name, self.deployment_group_name
            )
        )

        self.parsed_responses = self._get_parsed_responses(
            self.cluster_name,
            self.application_name,
            self.deployment_group_name,
        )

        expected_params = self._get_expected_params(
            self.service_arn,
            self.cluster_arn,
            self.application_name,
            self.deployment_group_name,
            expected_create_deployment_params,
        )

        stdout, _, _ = self.assert_params_list_for_cmd(
            cmdline, expected_params, 0
        )

        self.assertEqual(stdout, self.expected_stdout)

    def test_deploy_with_json_appspec(self):
        cmdline = self.PREFIX
        cmdline += '--service ' + self.service_name
        cmdline += ' --task-definition ' + self.task_def_file
        cmdline += ' --codedeploy-appspec ' + self.appspec_file_json

        expected_create_deployment_params = (
            self.mock_deployer._get_create_deploy_request(
                self.application_name, self.deployment_group_name
            )
        )

        self.parsed_responses = self._get_parsed_responses(
            self.cluster_name,
            self.application_name,
            self.deployment_group_name,
        )

        expected_params = self._get_expected_params(
            self.service_name,
            self.cluster_name,
            self.application_name,
            self.deployment_group_name,
            expected_create_deployment_params,
        )

        stdout, _, _ = self.assert_params_list_for_cmd(
            cmdline, expected_params, 0
        )

        self.assertEqual(stdout, self.expected_stdout)

    def test_deploy_with_custom_timeout(self):
        cmdline = self.PREFIX
        cmdline += '--service ' + self.service_name
        cmdline += ' --task-definition ' + self.task_def_file
        cmdline += ' --codedeploy-appspec ' + self.appspec_file

        expected_create_deployment_params = (
            self.mock_deployer._get_create_deploy_request(
                self.application_name, self.deployment_group_name
            )
        )

        custom_deployment_grp_response = {
            'deploymentGroupInfo': {
                'applicationName': self.application_name,
                'deploymentGroupName': self.deployment_group_name,
                'computePlatform': 'ECS',
                'blueGreenDeploymentConfiguration': {
                    'deploymentReadyOption': {'waitTimeInMinutes': 5},
                    'terminateBlueInstancesOnDeploymentSuccess': {
                        'terminationWaitTimeInMinutes': 60
                    },
                },
                'ecsServices': [
                    {
                        'serviceName': self.service_name,
                        'clusterName': self.cluster_name,
                    }
                ],
            }
        }
        custom_timeout = str(60 + 5 + TIMEOUT_BUFFER_MIN)

        self.parsed_responses = self._get_parsed_responses(
            self.cluster_name,
            self.application_name,
            self.deployment_group_name,
        )

        self.parsed_responses[2] = custom_deployment_grp_response

        expected_params = self._get_expected_params(
            self.service_name,
            self.cluster_name,
            self.application_name,
            self.deployment_group_name,
            expected_create_deployment_params,
        )

        expected_stdout = (
            "Successfully registered new ECS task "
            "definition " + self.task_definition_arn + "\n"
            "Successfully created deployment " + self.deployment_id + "\n"
            "Waiting for "
            + self.deployment_id
            + " to succeed (will wait up to "
            + custom_timeout
            + " minutes)...\nSuccessfully deployed "
            + self.task_definition_arn
            + " to service '"
            + self.service_name
            + "'\n"
        )

        stdout, _, _ = self.assert_params_list_for_cmd(
            cmdline, expected_params, 0
        )

        self.assertEqual(stdout, expected_stdout)

    def test_deploy_with_max_timeout(self):
        cmdline = self.PREFIX
        cmdline += '--service ' + self.service_name
        cmdline += ' --task-definition ' + self.task_def_file
        cmdline += ' --codedeploy-appspec ' + self.appspec_file

        expected_create_deployment_params = (
            self.mock_deployer._get_create_deploy_request(
                self.application_name, self.deployment_group_name
            )
        )

        custom_deployment_grp_response = {
            'deploymentGroupInfo': {
                'applicationName': self.application_name,
                'deploymentGroupName': self.deployment_group_name,
                'computePlatform': 'ECS',
                'blueGreenDeploymentConfiguration': {
                    'deploymentReadyOption': {'waitTimeInMinutes': 90},
                    'terminateBlueInstancesOnDeploymentSuccess': {
                        'terminationWaitTimeInMinutes': 300
                    },
                },
                'ecsServices': [
                    {
                        'serviceName': self.service_name,
                        'clusterName': self.cluster_name,
                    }
                ],
            }
        }
        max_timeout = str(MAX_WAIT_MIN)

        self.parsed_responses = self._get_parsed_responses(
            self.cluster_name,
            self.application_name,
            self.deployment_group_name,
        )

        self.parsed_responses[2] = custom_deployment_grp_response

        expected_params = self._get_expected_params(
            self.service_name,
            self.cluster_name,
            self.application_name,
            self.deployment_group_name,
            expected_create_deployment_params,
        )

        expected_stdout = (
            "Successfully registered new ECS task "
            "definition " + self.task_definition_arn + "\n"
            "Successfully created deployment " + self.deployment_id + "\n"
            "Waiting for "
            + self.deployment_id
            + " to succeed (will wait up to "
            + max_timeout
            + " minutes)...\nSuccessfully deployed "
            + self.task_definition_arn
            + " to service '"
            + self.service_name
            + "'\n"
        )

        stdout, _, _ = self.assert_params_list_for_cmd(
            cmdline, expected_params, 0
        )

        self.assertEqual(stdout, expected_stdout)

    def test_deploy_with_optional_values(self):
        custom_app = 'myOtherApp'
        custom_dgp = 'myOtherDgp'
        custom_cluster = 'myOtherCluster'

        cmdline = self.PREFIX
        cmdline += '--service ' + self.service_name
        cmdline += ' --task-definition ' + self.task_def_file
        cmdline += ' --codedeploy-appspec ' + self.appspec_file
        cmdline += ' --cluster ' + self.cluster_name
        cmdline += ' --codedeploy-application ' + custom_app
        cmdline += ' --codedeploy-deployment-group ' + custom_dgp
        cmdline += ' --cluster ' + custom_cluster

        expected_create_deployment_params = (
            self.mock_deployer._get_create_deploy_request(
                custom_app, custom_dgp
            )
        )

        self.parsed_responses = self._get_parsed_responses(
            custom_cluster, custom_app, custom_dgp
        )

        expected_params = self._get_expected_params(
            self.service_name,
            custom_cluster,
            custom_app,
            custom_dgp,
            expected_create_deployment_params,
        )

        stdout, _, _ = self.assert_params_list_for_cmd(
            cmdline, expected_params, 0
        )

        self.assertEqual(stdout, self.expected_stdout)

    def test_deploy_error_missing_appspec_property(self):
        cmdline = self.PREFIX
        cmdline += '--service ' + self.service_name
        cmdline += ' --task-definition ' + self.task_def_file
        cmdline += ' --codedeploy-appspec ' + self.missing_properties_appspec

        self.parsed_responses = [
            {
                'services': [
                    {
                        'serviceArn': self.service_arn,
                        'serviceName': self.service_name,
                        'clusterArn': 'arn:aws:ecs:::cluster/'
                        + self.cluster_name,
                    }
                ]
            },
            {
                'application': {
                    'applicationId': '876uyh6-45tdfg',
                    'applicationName': self.application_name,
                    'computePlatform': 'ECS',
                }
            },
            {
                'deploymentGroupInfo': {
                    'applicationName': self.application_name,
                    'deploymentGroupName': self.deployment_group_name,
                    'computePlatform': 'ECS',
                    'blueGreenDeploymentConfiguration': {
                        'deploymentReadyOption': {'waitTimeInMinutes': 5},
                        'terminateBlueInstancesOnDeploymentSuccess': {
                            'terminationWaitTimeInMinutes': 10
                        },
                    },
                    'ecsServices': [
                        {
                            'serviceName': self.service_name,
                            'clusterName': self.cluster_name,
                        }
                    ],
                }
            },
            {
                'taskDefinition': {
                    'taskDefinitionArn': self.task_definition_arn,
                    'family': 'test',
                    'containerDefinitions': [],
                }
            },
        ]

        expected_params = [
            {
                'operation': 'DescribeServices',
                'params': {
                    'cluster': self.cluster_name,
                    'services': [self.service_name],
                },
            },
            {
                'operation': 'GetApplication',
                'params': {'applicationName': self.application_name},
            },
            {
                'operation': 'GetDeploymentGroup',
                'params': {
                    'applicationName': self.application_name,
                    'deploymentGroupName': self.deployment_group_name,
                },
            },
            {
                'operation': 'RegisterTaskDefinition',
                'params': self.TASK_DEFINITION_JSON,
            },
        ]

        expected_stdout = (
            "Successfully registered new ECS task "
            "definition " + self.task_definition_arn + "\n"
        )

        expected_stderr = (
            "\naws: [ERROR]: Resource 'properties' must "
            "include property 'taskDefinition'\n"
        )

        stdout, stderr, _ = self.assert_params_list_for_cmd(
            cmdline, expected_params, 255
        )

        self.assertEqual(stdout, expected_stdout)
        self.assertEqual(stderr, expected_stderr)

    def test_deploy_error_invalid_platform(self):
        cmdline = self.PREFIX
        cmdline += '--service ' + self.service_name
        cmdline += ' --task-definition ' + self.task_def_file
        cmdline += ' --codedeploy-appspec ' + self.missing_properties_appspec

        self.parsed_responses = [
            {
                'services': [
                    {
                        'serviceArn': self.service_arn,
                        'serviceName': self.service_name,
                        'clusterArn': 'arn:aws:ecs:::cluster/'
                        + self.cluster_name,
                    }
                ]
            },
            {
                'application': {
                    'applicationId': '876uyh6-45tdfg',
                    'applicationName': self.application_name,
                    'computePlatform': 'Server',
                }
            },
            {
                'deploymentGroupInfo': {
                    'applicationName': self.application_name,
                    'deploymentGroupName': self.deployment_group_name,
                    'computePlatform': 'ECS',
                    'ecsServices': [
                        {
                            'serviceName': self.service_name,
                            'clusterName': self.cluster_name,
                        }
                    ],
                }
            },
        ]

        expected_params = [
            {
                'operation': 'DescribeServices',
                'params': {
                    'cluster': self.cluster_name,
                    'services': [self.service_name],
                },
            },
            {
                'operation': 'GetApplication',
                'params': {'applicationName': self.application_name},
            },
            {
                'operation': 'GetDeploymentGroup',
                'params': {
                    'applicationName': self.application_name,
                    'deploymentGroupName': self.deployment_group_name,
                },
            },
        ]

        expected_stderr = (
            "\naws: [ERROR]: Application '"
            + self.application_name
            + "' must support 'ECS' compute platform\n"
        )

        stdout, stderr, _ = self.assert_params_list_for_cmd(
            cmdline, expected_params, 255
        )

        self.assertEqual('', stdout)
        self.assertEqual(expected_stderr, stderr)

    def assert_params_list_for_cmd(
        self, cmd, params, expected_rc=0, stderr_contains=None
    ):
        stdout, stderr, rc = self.run_cmd(cmd, expected_rc)
        if stderr_contains is not None:
            self.assertIn(stderr_contains, stderr)

        self.assertEqual(len(self.operations_called), len(params))
        for i, param in enumerate(params):
            self.assertEqual(
                self.operations_called[i][0].name, param['operation']
            )
            self.assertEqual(self.operations_called[i][1], param['params'])

        return stdout, stderr, rc

    def _get_expected_params(
        self,
        service_name,
        cluster_name,
        app_name,
        dgp_name,
        create_deployment_params,
    ):
        return [
            {
                'operation': 'DescribeServices',
                'params': {
                    'cluster': cluster_name,
                    'services': [service_name],
                },
            },
            {
                'operation': 'GetApplication',
                'params': {'applicationName': app_name},
            },
            {
                'operation': 'GetDeploymentGroup',
                'params': {
                    'applicationName': app_name,
                    'deploymentGroupName': dgp_name,
                },
            },
            {
                'operation': 'RegisterTaskDefinition',
                'params': self.TASK_DEFINITION_JSON,
            },
            {
                'operation': 'CreateDeployment',
                'params': create_deployment_params,
            },
            {
                'operation': 'GetDeployment',
                'params': {'deploymentId': self.deployment_id},
            },
        ]

    def _get_parsed_responses(self, cluster_name, app_name, dgp_name):
        return [
            {
                'services': [
                    {
                        'serviceArn': self.service_arn,
                        'serviceName': self.service_name,
                        'clusterArn': 'arn:aws:ecs:::cluster/' + cluster_name,
                    }
                ]
            },
            {
                'application': {
                    'applicationId': '876uyh6-45tdfg',
                    'applicationName': app_name,
                    'computePlatform': 'ECS',
                }
            },
            {
                'deploymentGroupInfo': {
                    'applicationName': app_name,
                    'deploymentGroupName': dgp_name,
                    'computePlatform': 'ECS',
                    'blueGreenDeploymentConfiguration': {
                        'deploymentReadyOption': {'waitTimeInMinutes': 5},
                        'terminateBlueInstancesOnDeploymentSuccess': {
                            'terminationWaitTimeInMinutes': 10
                        },
                    },
                    'ecsServices': [
                        {
                            'serviceName': self.service_name,
                            'clusterName': cluster_name,
                        }
                    ],
                }
            },
            {
                'taskDefinition': {
                    'taskDefinitionArn': self.task_definition_arn,
                    'family': 'test',
                    'containerDefinitions': [],
                }
            },
            {'deploymentId': self.deployment_id},
            {
                'deploymentInfo': {
                    'applicationName': app_name,
                    'status': 'Succeeded',
                }
            },
        ]



# Copyright 2026 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from tests import BaseSessionTest


class TestPolly(BaseSessionTest):
    def test_start_speech_synthesis_stream(self):
        """StartSpeechSynthesisStream operation removed due to h2 requirement"""
        polly_client = self.session.create_client('polly', 'us-west-2')
        try:
            polly_client.start_speech_synthesis_stream
        except AttributeError:
            pass
        else:
            self.fail(
                'start_speech_synthesis_stream shouldn\'t be available on the '
                'polly client.'
            )



#!/usr/bin/env python
# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import copy
import unittest

from awscli.customizations.datapipeline import (
    ListRunsCommand,
    convert_described_objects,
)
from awscli.testutils import (
    BaseAWSCommandParamsTest,
    BaseAWSHelpOutputTest,
    mock,
)

API_DESCRIBE_OBJECTS = [
    {
        "fields": [
            {"key": "@componentParent", "refValue": "S3Input"},
            {
                "key": "@scheduledStartTime",
                "stringValue": "2013-08-19T20:00:00",
            },
            {"key": "parent", "refValue": "S3Input"},
            {"key": "@sphere", "stringValue": "INSTANCE"},
            {"key": "type", "stringValue": "S3DataNode"},
            {"key": "@version", "stringValue": "1"},
            {"key": "@status", "stringValue": "FINISHED"},
            {"key": "@actualEndTime", "stringValue": "2014-02-19T19:44:44"},
            {"key": "@actualStartTime", "stringValue": "2014-02-19T19:44:43"},
            {
                "key": "output",
                "refValue": "@MyCopyActivity_2013-08-19T20:00:00",
            },
            {"key": "@scheduledEndTime", "stringValue": "2013-08-19T21:00:00"},
        ],
        "id": "@S3Input_2013-08-19T20:00:00",
        "name": "@S3Input_2013-08-19T20:00:00",
    },
    {
        "fields": [
            {"key": "@componentParent", "refValue": "MyEC2Resource"},
            {"key": "@resourceId", "stringValue": "i-12345"},
            {
                "key": "@scheduledStartTime",
                "stringValue": "2013-08-19T23:00:00",
            },
            {"key": "parent", "refValue": "MyEC2Resource"},
            {"key": "@sphere", "stringValue": "INSTANCE"},
            {"key": "@attemptCount", "stringValue": "1"},
            {"key": "type", "stringValue": "Ec2Resource"},
            {"key": "@version", "stringValue": "1"},
            {"key": "@status", "stringValue": "CREATING"},
            {
                "key": "input",
                "refValue": "@MyCopyActivity_2013-08-19T23:00:00",
            },
            {"key": "@triesLeft", "stringValue": "2"},
            {"key": "@actualStartTime", "stringValue": "2014-02-19T19:59:45"},
            {
                "key": "@headAttempt",
                "refValue": "@MyEC2Resource_2013-08-19T23:00:00_Attempt=1",
            },
            {"key": "@scheduledEndTime", "stringValue": "2013-08-20T00:00:00"},
        ],
        "id": "@MyEC2Resource_2013-08-19T23:00:00",
        "name": "@MyEC2Resource_2013-08-19T23:00:00",
    },
]

JSON_FORMATTER_PATH = 'awscli.formatter.JSONFormatter.__call__'
LIST_FORMATTER_PATH = 'awscli.customizations.datapipeline.listrunsformatter.ListRunsFormatter.__call__'  # noqa


class TestConvertObjects(unittest.TestCase):
    def test_convert_described_objects(self):
        converted = convert_described_objects(API_DESCRIBE_OBJECTS)
        self.assertEqual(len(converted), 2)
        # This comes from a "refValue" value.
        self.assertEqual(converted[0]['@componentParent'], 'S3Input')
        # Should also merge in @id and name.
        self.assertEqual(converted[0]['@id'], "@S3Input_2013-08-19T20:00:00")
        self.assertEqual(converted[0]['name'], "@S3Input_2013-08-19T20:00:00")
        # This comes from a "stringValue" value.
        self.assertEqual(converted[0]['@sphere'], "INSTANCE")

    def test_convert_objects_are_sorted(self):
        describe_objects = copy.deepcopy(API_DESCRIBE_OBJECTS)
        # Change the existing @scheduledStartTime from
        # 20:00:00 to 23:59:00
        describe_objects[0]['fields'][1]['stringValue'] = "2013-08-19T23:59:00"
        converted = convert_described_objects(
            describe_objects,
            sort_key_func=lambda x: (x['@scheduledStartTime'], x['name']),
        )
        self.assertEqual(
            converted[0]['@scheduledStartTime'], '2013-08-19T23:00:00'
        )
        self.assertEqual(
            converted[1]['@scheduledStartTime'], '2013-08-19T23:59:00'
        )


class FakeParsedArgs:
    def __init__(self, **kwargs):
        self.endpoint_url = None
        self.region = None
        self.verify_ssl = None
        self.output = None
        self.query = None
        self.__dict__.update(kwargs)


class TestCommandsRunProperly(BaseAWSCommandParamsTest):
    def setUp(self):
        super(TestCommandsRunProperly, self).setUp()
        self.query_objects = mock.Mock()
        self.describe_objects = mock.Mock()
        self.client = mock.Mock()
        self.client.get_paginator.return_value = self.query_objects
        self.client.describe_objects = self.describe_objects

        self.driver.session = mock.Mock()
        self.driver.session.user_agent_extra = ''
        self.driver.session.emit_first_non_none_response.return_value = None
        self.driver.session.create_client.return_value = self.client
        self.query_objects.paginate.return_value.build_full_result.return_value = {
            'ids': ['object-ids']
        }
        self.describe_objects.return_value = {
            'pipelineObjects': API_DESCRIBE_OBJECTS
        }
        self.expected_response = convert_described_objects(
            API_DESCRIBE_OBJECTS,
            sort_key_func=lambda x: (x['@scheduledStartTime'], x['name']),
        )

    def test_list_runs(self):
        command = ListRunsCommand(self.driver.session)
        command(
            ['--pipeline-id', 'my-pipeline-id'],
            parsed_globals=FakeParsedArgs(region='us-east-1'),
        )
        self.assertTrue(self.query_objects.paginate.called)
        self.describe_objects.assert_called_with(
            pipelineId='my-pipeline-id', objectIds=['object-ids']
        )

    @mock.patch(JSON_FORMATTER_PATH)
    @mock.patch(LIST_FORMATTER_PATH)
    def test_list_runs_formatter_explicit_choice(
        self, list_formatter, json_formatter
    ):
        command = ListRunsCommand(self.driver.session)
        command(
            ['--pipeline-id', 'my-pipeline-id'],
            parsed_globals=FakeParsedArgs(region='us-east-1', output='json'),
        )
        json_formatter.assert_called_once_with(
            'list-runs', self.expected_response
        )
        self.assertFalse(list_formatter.called)

    @mock.patch(JSON_FORMATTER_PATH)
    @mock.patch(LIST_FORMATTER_PATH)
    def test_list_runs_formatter_implicit_choice(
        self, list_formatter, json_formatter
    ):
        command = ListRunsCommand(self.driver.session)
        command(
            ['--pipeline-id', 'my-pipeline-id'],
            parsed_globals=FakeParsedArgs(region='us-east-1'),
        )
        list_formatter.assert_called_once_with(
            'list-runs', self.expected_response
        )
        self.assertFalse(json_formatter.called)


class TestHelpOutput(BaseAWSHelpOutputTest):
    def test_list_runs_help_output(self):
        self.driver.main(['datapipeline', 'get-pipeline-definition', 'help'])
        self.assert_contains('pipeline definition')
        # The previous API docs should not be in the output
        self.assert_not_contains('pipelineObjects')



#!/usr/bin/env python
# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import base64
import json

from awscli.testutils import BaseAWSCommandParamsTest


class TestListObjects(BaseAWSCommandParamsTest):
    prefix = 's3api list-objects'

    def setUp(self):
        super(TestListObjects, self).setUp()
        self.parsed_response = {'Contents': []}

    def test_simple(self):
        cmdline = self.prefix
        cmdline += ' --bucket mybucket'
        self.assert_params_for_cmd(
            cmdline, {'Bucket': 'mybucket', 'EncodingType': 'url'}
        )

    def test_max_items(self):
        cmdline = self.prefix
        cmdline += ' --bucket mybucket'
        # The max-items is a customization and therefore won't
        # show up in the result params.
        cmdline += ' --max-items 100'
        self.assert_params_for_cmd(
            cmdline, {'Bucket': 'mybucket', 'EncodingType': 'url'}
        )

    def test_page_size(self):
        cmdline = self.prefix
        cmdline += ' --bucket mybucket'
        # The max-items is a customization and therefore won't
        # show up in the result params.
        cmdline += ' --page-size 100'
        self.assert_params_for_cmd(
            cmdline,
            {'Bucket': 'mybucket', 'MaxKeys': 100, 'EncodingType': 'url'},
        )

    def test_starting_token(self):
        # We don't need to test this in depth because botocore
        # tests this.  We just want to make sure this is hooked up
        # properly.
        cmdline = self.prefix
        cmdline += ' --bucket mybucket'
        token = {"Marker": "foo"}
        token = base64.b64encode(json.dumps(token).encode('utf-8'))
        token = token.decode('utf-8')
        cmdline += ' --starting-token %s' % token
        self.assert_params_for_cmd(
            cmdline,
            {'Bucket': 'mybucket', 'Marker': 'foo', 'EncodingType': 'url'},
        )

    def test_no_paginate(self):
        cmdline = self.prefix
        cmdline += ' --bucket mybucket --no-paginate'
        self.assert_params_for_cmd(
            cmdline, {'Bucket': 'mybucket', 'EncodingType': 'url'}
        )

    def test_max_keys_can_be_specified(self):
        cmdline = self.prefix
        # --max-keys is a hidden argument and not documented,
        # but for back-compat reasons if a user specifies this,
        # we will automatically see this and turn auto-pagination off.
        cmdline += ' --bucket mybucket --max-keys 1'
        self.assert_params_for_cmd(
            cmdline,
            {'Bucket': 'mybucket', 'MaxKeys': 1, 'EncodingType': 'url'},
        )
        self.assertEqual(len(self.operations_called), 1)
        self.assertEqual(len(self.operations_called), 1)
        self.assertEqual(self.operations_called[0][0].name, 'ListObjects')

    def test_pagination_params_cannot_be_supplied_with_no_paginate(self):
        cmdline = (
            self.prefix + ' --bucket mybucket --no-paginate ' '--max-items 100'
        )
        self.assert_params_for_cmd(
            cmdline,
            expected_rc=252,
            stderr_contains="Cannot specify "
            "--no-paginate along with pagination arguments: "
            "--max-items",
        )



# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License'). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the 'license' file accompanying this file. This file is
# distributed on an 'AS IS' BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from awscli.testutils import BaseAWSCommandParamsTest


class TestGetLoginPasswordCommand(BaseAWSCommandParamsTest):
    def setUp(self):
        super(TestGetLoginPasswordCommand, self).setUp()
        self.parsed_responses = [
            {
                'authorizationData': {
                    "authorizationToken": "Zm9vOmJhcg==",
                    "expiresAt": "2015-10-16T00:00:00Z",
                }
            },
        ]

    def test_prints_login_password(self):
        stdout = self.run_cmd("ecr-public get-login-password")[0]
        self.assertIn('bar', stdout)
        self.assertEqual(1, len(self.operations_called))
